/***************************************************************************
 *   Copyright (C) 2005 by Joachim Friedrich                               *
 *   joachim.friedrich@biozentrum.uni-wuerzburg.de                         *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute *
 *   it and/or modify it under the terms of the GNU General Public License *
 *   as published by the Free Software Foundation; either version 2 of the *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

// -*- C++ -*- generated by wxGlade 0.3.2 on Thu Jul 01 16:50:26 2004

#ifdef WINDOWS_SYSTEM
#undef DATADIR
#endif

#include <wx/wx.h>
#include <wx/image.h>

#include <wx/filename.h>
#include <wx/utils.h>
#include <wx/toolbar.h>
#include <wx/progdlg.h>
#include <stdexcept>

// Included for hyperlinks
#include "wxHyperlinksCtrl.h"

//#include <wx/stc/stc.h>  // styled text control

// begin wxGlade: ::dependencies
#include <wx/splitter.h>
#include <wx/treectrl.h>
#include <wx/notebook.h>
// end wxGlade


#include <wx/html/helpfrm.h>
#include <wx/html/helpctrl.h>

#include "tMatrix.h"
#include "aligncode.h"
#include "profile.h"
#include "profile_chooser.h"
#include "traits_chooser.h"
#include "parser.h"
#include "traits.hpp"

#include "error_output.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#define PROFDIST_SHARE DATADIR "/profdist/"
#else 
#define PROFDIST_SHARE
#endif 

//---------------------------------------------------------------------------
// DEFINES, MAKROS
//---------------------------------------------------------------------------
#if !defined(PROFDIST_H)
#define PROFDIST_H

#define USE_XPM_BITMAPS 1
#ifdef __WXMSW__
#define wxUSE_XPM_IN_MSW 1
#endif

#if USE_XPM_BITMAPS && defined(__WXMSW__) && !wxUSE_XPM_IN_MSW
#error You need to enable XPM support to use XPM bitmaps with toolbar!
#endif // USE_XPM_BITMAPS

#include "icons/open.xpm"
#include "icons/about.xpm"
#include "icons/save.xpm"
#include "icons/undo.xpm"
#include "icons/copy.xpm"
#include "icons/cut.xpm"
#include "icons/paste.xpm"
#include "icons/run.xpm"
#include "icons/tree.xpm"
#include "icons/pref.xpm"
#include "icons/help.xpm"
#include "icons/present.xpm"
#include "icons/arrowright.xpm"
#include "icons/arrowleft.xpm"


//---------------------------------------------------------------------------
// CLASS DEFINITIONS
//---------------------------------------------------------------------------

class MyApp : public wxApp
{
  private:
    static std::string examples_path;
    static std::string help_path;
  public:
    MyApp();
    bool OnInit();
    static std::string const& get_help_path();
    static std::string const& get_examples_path();
};

class MyDialogAbout : public wxDialog
{
  public:
    // begin wxGlade: MyDialogAbout::ids
    // end wxGlade

    MyDialogAbout( wxWindow * parent, int id, const wxString & title, const wxPoint & pos = wxDefaultPosition,
        const wxSize & size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE );

  private:
    // begin wxGlade: MyDialogAbout::methods
    void set_properties();
    void do_layout();
    // end wxGlade

    void onLinkWx( wxHyperlinkEvent & event );
    void onLinkBoost( wxHyperlinkEvent & event );

    DECLARE_EVENT_TABLE()

  protected :
      // begin wxGlade: MyDialogAbout::attributes
      wxStaticBitmap* bitmap_1;
      wxStaticText* label_5;
      wxHyperlinksCtrl* mHyperWx, *mHyperBoost;
      // end wxGlade

};




class MyDialogProfile : public wxDialog
{
  public:
    // begin wxGlade: MyDialogProfile::ids
    // end wxGlade

    MyDialogProfile( wxWindow * parent, int id, const wxString & title, const wxPoint & pos = wxDefaultPosition,
        const wxSize & size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE );

    void onBtnProfileOk( wxCommandEvent & WXUNUSED( event ) );
    void onBtnProfileCancel( wxCommandEvent & WXUNUSED( event ) );
    void onBtnProfileHelp( wxCommandEvent & WXUNUSED( event ) );
    wxString getProfile();

  private:
    // begin wxGlade: MyDialogProfile::methods
    void set_properties();
    void do_layout();
    wxHtmlHelpController help;
    // end wxGlade

    DECLARE_EVENT_TABLE()

  protected :
      // begin wxGlade: MyDialogProfile::attributes
      wxStaticBox* sizer_25_staticbox;
      wxStaticText* label_4;
      wxTextCtrl* txtProfileInput;
      wxButton* btnProfileOk;
      wxButton* btnProfileCancel;
      wxButton* btnProfHelp;
      // end wxGlade
};



class MyFrame : public wxFrame
{
  public:
    // begin wxGlade: MyFrame::ids
    // end wxGlade

    MyFrame( wxWindow * parent, int id, const wxString & title, const wxPoint & pos = wxDefaultPosition, const wxSize & size =
        wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE );

    bool loadXmlFile();
    //void getSelection();
    void setSelection();
    void saveXmlFile();

  private:
    void copy_text_window( wxTextCtrl * source, wxTextCtrl * destination ) const;
    void display_file( wxString const& tempfile ) const;
    // begin wxGlade: MyFrame::methods
    void set_properties();
    void do_layout();
    // end wxGlade

    wxHtmlHelpController help;

    DECLARE_EVENT_TABLE()

      void openFileBoot();
    void openFileDist();
    void openFileNj();
    void openFileCons();
    void openFilePnj();
	
    bool startBoot();
	
	template<typename Traits>
	bool startSpecializedBoot();
	
	bool startCons();
    bool startDist();
    
	template<typename Traits>
    bool startDistSingle(profdist::alignment&);
    
	template<typename Traits>
    bool startDistMultiple(profdist::alignment&);
    
	bool startNj();
    bool startNjSingle();
    bool startNjMultiple();
    bool startPnj();

    void saveBoot();
    void saveCons();
    void saveDist();
    void saveNj();
    void savePnj();

    void onTxtEnter( wxCommandEvent & event );

    void move_to_top( wxTextCtrl * t );

    void treeViewCons();
    void treeViewNj();
    void treeViewPnj();

    void onChkBootInput( wxCommandEvent & WXUNUSED( event ) );
    void onChkDistInput( wxCommandEvent & WXUNUSED( event ) );
    void onChkNjInput( wxCommandEvent & WXUNUSED( event ) );
    void onChkPnjInput( wxCommandEvent & WXUNUSED( event ) );

    void onChkBootOutput( wxCommandEvent & WXUNUSED( event ) );
    void onChkDistOutput( wxCommandEvent & WXUNUSED( event ) );
    void onChkNjOutput( wxCommandEvent & WXUNUSED( event ) );
    void onChkPnjOutput( wxCommandEvent & WXUNUSED( event ) );

    void onMnuOpen( wxCommandEvent & WXUNUSED( event ) );
    void onMnuSave( wxCommandEvent & WXUNUSED( event ) );
    void onMnuExit( wxCommandEvent & WXUNUSED( event ) );

    void onMnuUndo( wxCommandEvent & WXUNUSED( event ) );

    void onMnuCut( wxCommandEvent & WXUNUSED( event ) );
    void onMnuCopy( wxCommandEvent & WXUNUSED( event ) );
    void onMnuPaste( wxCommandEvent & WXUNUSED( event ) );

    void onMnuSelectAll( wxCommandEvent & WXUNUSED( event ) );

    void onMnuPref( wxCommandEvent & WXUNUSED( event ) );

    void onMnuStart( wxCommandEvent & WXUNUSED( event ) );
    void onMnuTreeView( wxCommandEvent & WXUNUSED( event ) );
    void onMnuGamma( wxCommandEvent & WXUNUSED( event ) );

    void onMnuHelp( wxCommandEvent & WXUNUSED( event ) );
    void onMnuAbout( wxCommandEvent & WXUNUSED( event ) );

    void onNotebook( wxNotebookEvent & event );
    void onCloseEvent( wxCloseEvent & event );

    size_t is_proteine();

    template<typename Traits>
    bool loadRateMatrix( typename Traits::rate_matrix  & Q );

    template<typename Traits>
    bool start_pnj_main_loop( profdist::Profile & profile, std::vector<std::string> const& seq_names, tree_types::profile_map & profiles_found, typename Traits::rate_matrix const& Q, ostream & out );
    bool create_profile_dialog( profdist::profile_set & profiles );

    bool create_group_dialog( profdist::alignment const& sequences, list<list<size_t> > & profiles,
        vector<string> & prof_names, list<list<string> >  & names );
    void show_tree( std::vector<string> const& sequence_names, Tree const& tree );

    wxTextCtrl * getFocusedText();

  protected:
    // begin wxGlade: MyFrame::attributes
    wxStaticBox* szPnjOutput_staticbox;
    wxStaticBox* szPnjInput_staticbox;
    wxStaticBox* szConsInput_staticbox;
    wxStaticBox* szConsOutput_staticbox;
    wxStaticBox* szNjOutput_staticbox;
    wxStaticBox* szNjInput_staticbox;
    wxStaticBox* sizer_11_copy_staticbox;
    wxStaticBox* sizer_10_copy_staticbox;
    wxStaticBox* sizer_11_staticbox;
    wxStaticBox* sizer_10_staticbox;
    wxMenuBar* frame_1_menubar;
    wxStatusBar* frame_1_statusbar;
    wxToolBar* frame_toolbar;
    wxTextCtrl* txtBootInput;
    wxRadioBox* rdoBootInput;
    wxCheckBox* chkBootInput;
    wxTextCtrl* txtBootOutput;
    wxRadioBox* rdoBootOutput;
    wxCheckBox* chkBootOutput;
    wxPanel* ntbBootstrap;
    wxTextCtrl* txtDistInput;
    wxRadioBox* rdoDistInput;
    wxCheckBox* chkDistMultiple;
    wxCheckBox* chkDistInput;
    wxTextCtrl* txtDistOutput;
    wxRadioBox* rdoDistOutput;
    wxCheckBox* chkDistOutput;
    wxPanel* ntbDistance;
    wxTextCtrl* txtNjInput;
    wxRadioBox* rdoNjInput;
    wxCheckBox* chkNjMultiple;
    wxCheckBox* chkNjInput;
    wxTextCtrl* txtNjOutput;
    wxRadioBox* rdoNjOutput;
    wxCheckBox* chkNjOutput;
    wxPanel* ntbSystematic;
    wxTextCtrl* txtConsInput;
    wxTextCtrl* txtConsOutput;
    wxRadioBox* rdoConsOutput;
    wxRadioBox* rdoConsInput;
    wxPanel* ntbConsense;
    wxTextCtrl* txtPnjInput;
    wxRadioBox* rdoPnjInput;
    wxCheckBox* chkPnjInput;
    wxTextCtrl* txtPnjOutput;
    wxRadioBox* rdoPnjOutput;
    wxCheckBox* chkPnjOutput;
    wxPanel* ntbPnj;
    wxNotebook* ntbAll;
    // end wxGlade


    // -------------------------------------------------------------------------
    // Functions
    // -------------------------------------------------------------------------
    wxString setOpenFileFormatAlignment();
    wxString setOpenFileFormatAlignmentBoot();
    wxString setSaveFileFormatAlignment();

    template<typename Traits>
    bool startPnjWithAlignment( profdist::AlignCode<Traits> & code, typename Traits::rate_matrix& Q );
//    bool startPnjWithAlignment( cAlignCode& code, profdist::rate_matrix const & Q );
    template<typename Traits>
    bool startPnjWithProfile( profdist::Profile & prof, ProfileDialog::profile_list const& profiles,
    vector<string> const& profile_names, ProfileDialog::name_list const& sequence_names, typename Traits::rate_matrix const& Q );

    // -------------------------------------------------------------------------
    // Member Variables
    // -------------------------------------------------------------------------
    char flags[4];
    long mNumberBoot;
    profdist::CorrectionModel mEvoModel;
    profdist::ExpMType mExpm;
    profdist::FZero mFMax;
    wxString mFileRateMatrix;
    long mPnjMethod;
    long mPnjBootstrapThreshold;
    long mPnjIdentityThreshold;
    long mPnjShowOC;
    long mTreeViewType;
    wxString mFileTreeViewer1;
    wxString mFileTreeViewer2;

    long mNtbSelection;
    long mRdoBootInput;
    long mRdoBootOutput;
    long mRdoDistInput;
    long mRdoDistOutput;
    long mRdoNjInput;
    long mRdoNjOutput;
    long mRdoConsOutput;
    long mRdoPnjInput;
    long mRdoPnjOutput;


    // File Path
    wxString mPnjFilePath;
    wxString mBootFilePath;
    wxString mNjFilePath;
    wxString mConsFilePath;
    wxString mDistFilePath;

};



class MyDialogPref : public wxDialog
{
  public:
    // begin wxGlade: MyDialogPref::ids
    // end wxGlade

    MyDialogPref( wxWindow * parent, int id, const wxString & title, const wxPoint & pos = wxDefaultPosition,
        const wxSize & size = wxDefaultSize, long style = wxDEFAULT_DIALOG_STYLE );

    void setPreferences( long & numberBoot, profdist::CorrectionModel const& evoModel, profdist::ExpMType const& expm, profdist::FZero const& fMax, wxString & fileRateMatrix,
        long & pnjMethod, long & pnjBootstrapThreshold, long & pnjIdentityThreshold, long & pnjShowOC, long & treeViewType,
        wxString & fileTreeViewer1, wxString & fileTreeViewer2 );

    void getPreferences( long & numberBoot, profdist::CorrectionModel& evoModel, profdist::ExpMType & expm, profdist::FZero & fMax, wxString & fileRateMatrix,
        long & pnjMethod, long & pnjBootstrapThreshold, long & pnjIdentityThreshold, long & pnjShowOC, long & treeViewType,
        wxString & fileTreeViewer1, wxString & fileTreeViewer2 );

    void onBtnPrefOpenTreeViewer1( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefOpenTreeViewer2( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefOpenRatematrix( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefOk( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefCancel( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefHelp( wxCommandEvent & WXUNUSED( event ) );
    void onBtnPrefExpert( wxCommandEvent & WXUNUSED( event ) );
    void onRdoPrefTree1( wxCommandEvent & WXUNUSED( event ) );
    void onRdoPrefTree2( wxCommandEvent & WXUNUSED( event ) );
    void onRdoPrefHypertree( wxCommandEvent & WXUNUSED( event ) );
    void onRdoPrefNone( wxCommandEvent & WXUNUSED( event ) );
    void onRdoPrefEvolution( wxCommandEvent & WXUNUSED( event ) );

  private:
    // begin wxGlade: MyDialogPref::methods
    void set_properties();
    void do_layout();
    // end wxGlade
    wxHtmlHelpController help;
    DECLARE_EVENT_TABLE()

  protected :
      // begin wxGlade: MyDialogPref::attributes
      wxStaticBox* szPrefTreeViewer_staticbox;
      wxStaticBox* pnj_settings_staticbox;
      wxStaticBox* embl_static_box;
      wxStaticBox* threshold_static_box;
      wxStaticBox* szPrefDistance_staticbox;
      wxStaticBox* ratematrix_staticbox;
      wxStaticBox* szPrefBootstrap_staticbox;
      wxStaticText* label_1, * rate_matrix_label;
      wxTextCtrl* txtPrefNumberBoot;
      wxRadioBox* rdoPrefEvolution;
      //wxRadioBox* rdoPrefMatrixExpm;
      wxRadioBox* rdoPrefMaximum;
      wxTextCtrl* txtPrefRatematrix;
      wxBitmapButton* btnPrefOpenRatematrix;
      wxRadioBox* rdoPrefPnjType;
      wxStaticText* label_2;
      wxTextCtrl* txtPrefProfBootValue;
      wxStaticText* label_3;
      wxTextCtrl* txtPrefProfPercentIdent;
      wxCheckBox* chkPrefShowOC;
      wxRadioButton* rdoPrefTree1;
      wxTextCtrl* txtPrefTreeViewer1;
      wxBitmapButton* btnPrefOpenTreeViewer1;
      wxRadioButton* rdoPrefTree2;
      wxTextCtrl* txtPrefTreeViewer2;
      wxBitmapButton* btnPrefOpenTreeViewer2;
      wxRadioButton* rdoPrefNoneTree;
      wxStaticText* text_label_1, * bootstrap_comment;
      wxButton* btnPrefOk;
      wxButton* btnPrefCancel;
      wxButton* btnPrefHelp;
      wxButton* btnPrefExpert;
      // end wxGlade

      long mPrefTreeType;
};



class MyTreeItemData : public wxTreeItemData
{
  public:

    MyTreeItemData()
    {
      mSeq = 0;
      mNumSeq = 0;
    }

    MyTreeItemData( wxString strLineage, int seq, wxString itemText )
    {
      mLineage = strLineage;
      mSeq = seq;
      mItemText = itemText;
      mNumSeq = 1;
    }

    //void ShowInfo( wxTreeCtrl * tree );

    wxString getLineage()
    {
      return mLineage;
    }

    int getSeq()
    {
      return mSeq;
    }

    int getNumSeq()
    {
      return mNumSeq;
    }

    wxString getText()
    {
      return mItemText;
    }

    void iterNumSeq()
    {
      ++mNumSeq;
    }

  private:
    wxString mLineage;
    wxString mItemText;
    int mSeq;
    int mNumSeq;
};

enum //mainFrame
{
  // MyFrame
  idMnuOpen, //
       idMnuSave, //
       idMnuExit, //
       idMnuUndo, //
       idMnuCopy, //
       idMnuCut, //
       idMnuPaste, //
       idMnuSelectAll, //
       idMnuPref, //
       idMnuStart, //
       idMnuTreeView, //
       idMnuGamma, //
       idMnuHelp, //
       idMnuAbout, //

       idNtbAll, //
       idNtbBootstrap, //
       idNtbDistance, //
       idNtbNj, //
       idNtbConsense, //
       idNtbPnj, //

       idChkBootInput, //
       idChkDistInput, //
       idChkNjInput, //
       idChkPnjInput, //

       idChkBootOutput, //
       idChkDistOutput, //
       idChkNjOutput, //
       idChkConsOutput, //
       idChkPnjOutput, //

       idChkDistMultiple, //
       idChkNjMultiple, //

       // MyDialogPref
       idBtnPrefOpenTreeViewer1, //
       idBtnPrefOpenTreeViewer2, //
       idBtnPrefOpenRatematrix, //
       idBtnPrefOk, //
       idBtnPrefCancel, //
       idBtnPrefHelp, //
       idBtnPrefExpert, //
       idRdoPrefTree1, //
       idRdoPrefTree2, //
       idRdoPrefHypertree, //
       idRdoPrefNone, //
       idRdoPrefEvolution, //

       // MyDialogProfile
       idBtnProfileOk, //
       idBtnProfileCancel, //
       idBtnProfileHelp, //

       // MyDialogProfile
       idBtnGroupRight, //
       idBtnGroupLeft, //
       idBtnGroupOk, //
       idBtnGroupCancel, //
       idBtnGroupHelp, //

       idTreeOrganism, //
       idTreeProfiles, //
       idMnuGroupRename, //

       // MyDialogAbout
       idLinkWx,
       idLinkBoost
};

#endif // PROFDIST_H

