/***************************************************************************
 *   Copyright (C) 2005 by Joachim Friedrich                               *
 *   joachim.friedrich@biozentrum.uni-wuerzburg.de                         *
 *   and Andreas Pokorny andreas.pokorny@biozentrum.uni-wuerzburg.de       *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute * 
 *   it and/or modify it under the terms of the GNU General Public License * 
 *   as published by the Free Software Foundation; either version 2 of the * 
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <wx/wx.h>
#include <iostream>
#include <wx/msgdlg.h>
#include <string>
#include <locale>

#include "error_output.h"
#include "converter.h"

#if ! wxUSE_MSGDLG
#error broken wx installation .. ?
#endif

void error_output( wxWindow *p, const std::exception & e )
{
  wxString message( convert(e.what()) );
  
  wxMessageDialog box( p, message, _T("Error"), wxICON_ERROR );
  box.ShowModal();
  box.Close();
}

void error_output( wxWindow *p, std::string const & error )
{
  wxString message( convert( error.c_str() ));
  
  wxMessageDialog box( p, message, _T("Error"), wxICON_ERROR );
  box.ShowModal();
  box.Close();
}




void error_output( wxWindow *p, const error & e )
{
  wxString message;
  switch ( e )
  {
    case singMat1:
      message = _T("Singular matrix -> Not possible to calculate inverse! #1. You could try a different correction method");
    break;
    case singMat2:
      message = _T("Singular matrix -> Not possible to calculate inverse! #2. You could try a different correction method");
    break;
    case deriv1:
      message = _T("Division by zero! First Derivation of LogLike!\nThe alignment might be to divergent. You could try a different correction method");
    break;
    case deriv2:
      message = _T("Division by zero! Second Derivation of LogLike!\nThe alignment might be to divergent. You could try a different correction method");
    break;
    case matMul:
      message = _T("Matrices do not conform for multiplication!\nYou discovered a bug in our application, blame us!");
    break;
    case matDet:
      message = _T("Determinat of non-square Matrix not defined!\nYou discovered a bug in our application, blame us!");
    break;
    case matIndexI:
      message =  _T("Matrix operator(i): index out of range\nYou discovered a bug in our application, blame us!");
    break;
    case matIndexIJ:
      message = _T("Matrix operator(i,j): index out of range\nYou discovered a bug in our application, blame us!");
    break;
    case matAdd:
      message = _T("Matrices do not conform in addition\nYou discovered a bug in our application, blame us!");
    break;
    case matSubst:
      message = _T("Matrices do not conform in subtraction\nYou discovered a bug in our application, blame us!");
    break;
    case tZero:
      message = _T("expm(t*Q) with t = 0.\nThe alignment might be to divergent. You could try a different correction method.");
    break;
    case seqLength:
      message = _T("The sequences have not the same size!\nProfdist only processes alignments.");
    break;
    case kimura1:
      message = _T("Term for logarithm is less/equal to zero! (Kimura)\nThe alignment might be to divergent. You could try a different correction method");
    break;
    case kimura2:
      message = _T("Term for logarithm is less/equal to zero! (Kimura)\nThe alignment might be to divergent. You could try a different correction method");
    break;
  }
  wxMessageDialog box( p, message, _T("Error"), wxICON_ERROR );
  box.ShowModal();
  box.Close();
}

