/***************************************************************************
 *   Copyright (C) 2005 by Andreas Pokorny                                 *
 *   andreas.pokorny@biozentrum.uni-wuerzburg.de                           *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute * 
 *   it and/or modify it under the terms of the GNU General Public License * 
 *   as published by the Free Software Foundation; either version 2 of the * 
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/trim.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/lexical_cast.hpp>
#include <wx/wx.h>
#include "icons/arrowright.xpm"
#include "profile_chooser.h"
#include "converter.h"

using namespace boost::algorithm;
using namespace std;


BEGIN_EVENT_TABLE( ProfileDialog, wxDialog ) 
  EVT_BUTTON( ProfileDialog::SubmitProfile, ProfileDialog::on_submit_profile)
  EVT_BUTTON( ProfileDialog::Ok, ProfileDialog::on_ok )
  EVT_BUTTON( ProfileDialog::Cancel, ProfileDialog::on_cancel)
  EVT_BUTTON( ProfileDialog::ResetData, ProfileDialog::on_reset)
  EVT_BUTTON( ProfileDialog::Help, ProfileDialog::on_help )

  EVT_TREE_ITEM_RIGHT_CLICK( ProfileDialog::Organism, ProfileDialog::OnItemRightClick )
  EVT_TREE_ITEM_RIGHT_CLICK( ProfileDialog::Profiles, ProfileDialog::OnItemRightClick )

END_EVENT_TABLE() // end


TreeItemData::TreeItemData()
  : seq(0), num_seq(0) {}

TreeItemData::TreeItemData( wxString const& l, std::size_t s, wxString const& t )
  : lineage(l), item_text(t), seq(s), num_seq(1) {}


ProfileDialog::ProfileDialog( wxWindow * parent, int id, wxString const & title, ProfileFlag f, 
        profdist::alignment const& sequences,
    wxPoint const& pos, 
    wxSize const & size, long style ) :
  wxDialog( parent, id, title, pos, size,
      wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxSYSTEM_MENU | wxMAXIMIZE_BOX | wxMINIMIZE_BOX ), flags(f), num_profiles(0)
{
  construct_ui();
  create_tree(sequences);
}
ProfileDialog::ProfileDialog( wxWindow * parent, int id, wxString const & title, ProfileFlag f, wxPoint const& pos, 
    wxSize const & size, long style ) :
  wxDialog( parent, id, title, pos, size,
      wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxSYSTEM_MENU | wxMAXIMIZE_BOX | wxMINIMIZE_BOX ), flags(f), num_profiles(0)
{
  construct_ui();
}

void ProfileDialog::construct_ui() {

  window_1 = new wxSplitterWindow(this, -1, wxDefaultPosition, wxDefaultSize, wxSP_3D|wxSP_BORDER);
  panel_2 = new wxPanel(window_1, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL);
  window_1_pane_1 = new wxPanel(window_1, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL);
  panel_1 = new wxPanel(window_1_pane_1, -1, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER);
  organism_tree = new wxTreeCtrl(window_1_pane_1, Organism, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS|wxTR_LINES_AT_ROOT|wxTR_MULTIPLE|wxTR_MULTIPLE|wxTR_DEFAULT_STYLE|wxNO_BORDER);
  submit_profile_button = new wxBitmapButton(panel_1, SubmitProfile, wxBitmap( arrowright_xpm ));
  profiles_tree = new wxTreeCtrl(panel_2, Profiles, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS|wxTR_LINES_AT_ROOT|wxTR_EDIT_LABELS|wxTR_MULTIPLE|wxTR_MULTIPLE|wxTR_DEFAULT_STYLE|wxNO_BORDER);
  chkGroupLeft = new wxCheckBox(this, -1, _("Build Profile for each Sequence on left side"));
  reset_button = new wxButton(this, ResetData, _("Reset"));
  ok_button = new wxButton(this, Ok, _("OK"));
  cancel_button = new wxButton(this, Cancel, _("Cancel"));
#ifndef DARWIN_SYSTEM
  help_button = new wxButton(this, Help, _("Help"));
#endif

  set_properties();
  do_layout();
}


void ProfileDialog::set_properties()
{
  // begin wxGlade: ProfileDialog::set_properties
  SetTitle(_("Profile Definition"));
  SetSize(wxSize(694, 567));
  submit_profile_button->SetSize(wxSize(23, 23));
  ok_button->SetFocus();
  ok_button->SetDefault();
  // end wxGlade
}


void ProfileDialog::do_layout()
{
  // begin wxGlade: ProfileDialog::do_layout
  wxBoxSizer* sizer_36 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_37 = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer* sizer_24_copy_copy = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_27 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_18 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_41 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_38 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_35 = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer* sizer_39 = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer* sizer_40 = new wxBoxSizer(wxVERTICAL);
  sizer_36->Add(5, 5, 0, 0, 0);
  sizer_37->Add(5, 5, 0, 0, 0);
  sizer_40->Add(organism_tree, 1, wxEXPAND, 0);
  sizer_38->Add(sizer_40, 1, wxEXPAND, 0);
  sizer_35->Add(5, 5, 0, 0, 0);
  sizer_39->Add(20, 20, 0, 0, 0);
  sizer_39->Add(submit_profile_button, 0, 0, 0);
  sizer_39->Add(10, 10, 0, 0, 0);
  sizer_35->Add(sizer_39, 1, wxEXPAND, 0);
  sizer_35->Add(5, 5, 0, 0, 0);
  panel_1->SetAutoLayout(true);
  panel_1->SetSizer(sizer_35);
  sizer_35->Fit(panel_1);
  sizer_35->SetSizeHints(panel_1);
  sizer_38->Add(panel_1, 0, wxEXPAND, 0);
  window_1_pane_1->SetAutoLayout(true);
  window_1_pane_1->SetSizer(sizer_38);
  sizer_38->Fit(window_1_pane_1);
  sizer_38->SetSizeHints(window_1_pane_1);
  sizer_41->Add(profiles_tree, 1, wxEXPAND, 0);
  panel_2->SetAutoLayout(true);
  panel_2->SetSizer(sizer_41);
  sizer_41->Fit(panel_2);
  sizer_41->SetSizeHints(panel_2);
  window_1->SplitVertically(window_1_pane_1, panel_2, 357);
  sizer_37->Add(window_1, 1, wxEXPAND, 0);
  sizer_37->Add(5, 5, 0, 0, 0);
  sizer_18->Add(chkGroupLeft, 0, wxALIGN_CENTER_VERTICAL, 0);
  sizer_24_copy_copy->Add(sizer_18, 1, wxEXPAND, 0);
  sizer_27->Add(reset_button, 0, 0, 0);
  sizer_27->Add(5, 5, 0, 0, 0);
  sizer_27->Add(ok_button, 0, 0, 0);
  sizer_27->Add(5, 5, 0, 0, 0);
  sizer_27->Add(cancel_button, 0, 0, 0);
#ifndef DARWIN_SYSTEM
  sizer_27->Add(5, 5, 0, 0, 0);
  sizer_27->Add(help_button, 0, wxALIGN_RIGHT, 0);
#endif
  sizer_24_copy_copy->Add(sizer_27, 0, wxALIGN_RIGHT, 0);
  sizer_37->Add(sizer_24_copy_copy, 0, wxEXPAND, 0);
  sizer_37->Add(5, 5, 0, 0, 0);
  sizer_36->Add(sizer_37, 1, wxEXPAND, 0);
  sizer_36->Add(5, 5, 0, 0, 0);
  SetAutoLayout(true);
  SetSizer(sizer_36);
  Layout();
  Centre();
  // end wxGlade
}
void ProfileDialog::on_ok( wxCommandEvent & WXUNUSED( event ) )
{
  EndModal( Ok );
}

void ProfileDialog::get_profiles( ProfileDialog::profile_list & profiles, vector<string> & profile_names,  name_list & sequence_names )
{
  wxTreeItemIdValue cookie;
  wxTreeItemId base_item = profiles_tree->GetFirstChild(  profiles_tree->GetRootItem(), cookie );

  while ( base_item != wxTreeItemId(wxTreeItemIdValue(0)) )
  {
    profile_names.push_back( convert_back( profiles_tree->GetItemText( base_item ).ToAscii() ) );
    list<size_t> single_profile;
    list<string> names_of_profile;

    if ( profiles_tree->ItemHasChildren( base_item ) )
      get_sub_profiles( base_item, single_profile, names_of_profile );
    else
    {
      TreeItemData * item = reinterpret_cast<TreeItemData *>( profiles_tree->GetItemData( base_item ) ); //
      single_profile.push_back( item->get_seq() );
      names_of_profile.push_back( convert_back( profiles_tree->GetItemText( base_item ).ToAscii() ) );
    }
    profiles.push_back( single_profile );
    sequence_names.push_back( names_of_profile);

    base_item = profiles_tree->GetNextChild( profiles_tree->GetRootItem(), cookie );
  }

  if ( chkGroupLeft->IsChecked() ) {
    left_profiles(organism_tree->GetRootItem(), profiles, profile_names, sequence_names);
  }
}
void ProfileDialog::get_sub_profiles( wxTreeItemId parent, list<size_t>  & profile, list<string> & seq_name )
{
  wxTreeItemIdValue cookie;
  wxTreeItemId child_item;

  child_item = profiles_tree->GetFirstChild( parent, cookie );

  while( child_item.IsOk() )
  {
    if ( profiles_tree->ItemHasChildren( child_item ) ){
      get_sub_profiles( child_item, profile, seq_name );
    }
    else
    {
      TreeItemData * item = reinterpret_cast<TreeItemData *>( profiles_tree->GetItemData( child_item ) ); 
      profile.push_back( item->get_seq() );
      seq_name.push_back( convert_back( profiles_tree->GetItemText( child_item).ToAscii() ));
    }
    child_item = profiles_tree->GetNextChild( parent, cookie );
  }
}

void ProfileDialog::left_profiles( wxTreeItemId parent,list<list<size_t> > & profiles, vector<string> & profile_names,  list<list<string> > & sequence_names ) 
{
  wxTreeItemIdValue cookie;
  wxTreeItemId child_item;

  child_item = organism_tree->GetFirstChild( parent, cookie );

  while( child_item.IsOk() )
  {
    if ( organism_tree->ItemHasChildren( child_item ) ) {
      left_profiles( child_item, profiles, profile_names, sequence_names);
    }
    else
    {
      list<size_t> single_prof;
      list<string> seq_name;

      TreeItemData * data = reinterpret_cast<TreeItemData *>(organism_tree->GetItemData( child_item ) );
      single_prof.push_back( data->get_seq() );
      seq_name.push_back( convert_back( organism_tree->GetItemText( child_item).ToAscii() ) );

      profile_names.push_back( convert_back( organism_tree->GetItemText( child_item).ToAscii() ) );
      profiles.push_back( single_prof );
      sequence_names.push_back( seq_name );
    }
    child_item = organism_tree->GetNextChild( parent, cookie );
  }
}


void ProfileDialog::on_submit_profile( wxCommandEvent & WXUNUSED( event ) )
{
  if( flags == MultiSelect ) 
  {
    wxArrayTreeItemIds organism_selection;
    std::size_t num_selections = organism_tree->GetSelections( organism_selection );

    if( num_selections )
    {
      wxTreeItemId root_item = profiles_tree->GetRootItem();
      ++num_profiles;

      if( num_selections != 1 ) {
        std::size_t num_leafs = normalize_array( organism_selection );
        root_item = profiles_tree->AppendItem( root_item, wxString(_T("profile ")) << num_profiles << _T(" [") << num_leafs << _T("]") );
      }
      reduce_seqs( organism_selection );
      

      for( int i = 0; i < organism_selection.GetCount(); ++i )
      {
        wxTreeItemId item_to_add = organism_selection.Item( i );
        TreeItemData * item = reinterpret_cast<TreeItemData*>(profiles_tree->GetItemData( item_to_add ));

        wxString name = organism_tree->GetItemText( item_to_add );
        wxTreeItemId item_added = profiles_tree->AppendItem(root_item, name, -1, -1, new TreeItemData( *item ) );// item->get_lineage(), item->get_seq(), name ) );
        profiles_tree->EnsureVisible( item_added );

        if( organism_tree->ItemHasChildren( item_to_add ) )
          submit_subtree( item_to_add, item_added );
        
        wxTreeItemId item_parent = organism_tree->GetItemParent( item_to_add );
        organism_tree->Delete( item_to_add );

        if( !organism_tree->ItemHasChildren( item_parent ) )
          deleteEmptyParent( item_parent );
      }

     // profiles_tree->Expand( root_item );
  //   profiles_tree->EnsureVisible( root_item );
    }
  }
  else {
  }
}

void ProfileDialog::submit_subtree( wxTreeItemId & organism_item, wxTreeItemId & profile_item )
{
  wxTreeItemIdValue cookie;
  for( wxTreeItemId it = organism_tree->GetFirstChild( organism_item, cookie )
      ; it != wxTreeItemId(wxTreeItemIdValue(0))
      ; it = organism_tree->GetNextChild( organism_item, cookie )
     ) {
    wxTreeItemId item_added = profiles_tree->AppendItem( profile_item 
        ,  organism_tree->GetItemText( it )
        , -1
        , -1
        , new TreeItemData( *reinterpret_cast<TreeItemData*>(organism_tree->GetItemData(it)) ) 
        );


    if( organism_tree->ItemHasChildren( it ) )
      submit_subtree( it, item_added );

//    profiles_tree->EnsureVisible( item_added );
  }
  organism_tree->DeleteChildren( organism_item );
}

void ProfileDialog::deleteEmptyParent( wxTreeItemId & item )
{
  wxTreeItemId itemParent;

  itemParent = organism_tree->GetItemParent( item );

  if ( item != organism_tree->GetRootItem() )
    organism_tree->Delete( item );

  if( itemParent != organism_tree->GetRootItem() 
      && organism_tree->ItemHasChildren( organism_tree->GetRootItem() ) )
    if( !organism_tree->ItemHasChildren( itemParent ) )
      deleteEmptyParent( itemParent );
}


void ProfileDialog::on_cancel( wxCommandEvent & WXUNUSED( event ) )
{
  EndModal( Cancel );
}

void ProfileDialog::on_reset( wxCommandEvent & WXUNUSED( event ) )
{
  EndModal( ResetData );
}

void ProfileDialog::on_help( wxCommandEvent & WXUNUSED( event ) )
{
  EndModal( Help );
}

bool ProfileDialog::is_child( wxTreeItemId const& parent, wxTreeItemId const& child )
{
  if( parent == child ) return true;
  if( organism_tree->ItemHasChildren( parent ) )  {
    wxTreeItemIdValue cookie;
    for( wxTreeItemId it = organism_tree->GetFirstChild( parent , cookie )
        ; it != wxTreeItemId(wxTreeItemIdValue(0))
        ; it = organism_tree->GetNextChild( parent, cookie )
       ) 
      if( it == child || is_child( it, child) ) return true;
  }
  return false;
}

void ProfileDialog::reduce_seqs(wxArrayTreeItemIds & ids){
  for( std::size_t i = 0; i < ids.GetCount(); ++ i ){
    wxTreeItemId item_id = ids.Item(i)
      , parent_id = organism_tree->GetItemParent(item_id );
    TreeItemData * item = reinterpret_cast<TreeItemData *>( organism_tree->GetItemData( item_id ) ); //
    TreeItemData * parent_item = reinterpret_cast<TreeItemData*>( organism_tree->GetItemData( parent_id  ) );
    while( item && parent_item ) {
      parent_item->set_num_seq( parent_item->get_num_seq() - item->get_num_seq() );
      wxString item_text = parent_item->get_lineage();
      item_text << _T(" [") << parent_item->get_num_seq() << _T("]");
      organism_tree->SetItemText( parent_id, item_text );

      parent_id = organism_tree->GetItemParent(parent_id);
      parent_item = reinterpret_cast<TreeItemData*>( organism_tree->GetItemData( parent_id  ) );
    }
  }
    
}
std::size_t ProfileDialog::normalize_array(wxArrayTreeItemIds & ids)
{
  std::size_t ret = 0;
  for( std::size_t i = 0; i < ids.GetCount(); ++ i ){
    TreeItemData * item = reinterpret_cast<TreeItemData *>( organism_tree->GetItemData( ids.Item(i) ) ); //
    ret += item->get_num_seq();
    
    if( organism_tree->ItemHasChildren( ids.Item(i) ) ) 
      for( std::size_t j = 0; j < ids.GetCount() ; ++j ) 
        if( i != j && is_child( ids.Item(i), ids.Item(j) ) )
          ids.RemoveAt( j-- );
  }
  return ret;
}

void ProfileDialog::create_tree( profdist::alignment const& sequences )
{
 
  using namespace profdist;
  organism_tree->DeleteAllItems();
  profiles_tree->DeleteAllItems();
  std::size_t sequence_index = 0, position_index = 0;

  wxTreeItemId organism_item = organism_tree->AddRoot( _T("Organism Classification") );
  profiles_tree->AddRoot( _T("Profile") );
  bool first_elem = true;

  for( alignment::const_iterator it = sequences.begin(), e = sequences.end(); 
      it != e; ++ it, ++sequence_index)
  {
    wxTreeItemId insertion_point = organism_item;
    std::list<alignment::value_type::string_range>::const_iterator class_it = it->classification.begin();

    for( std::list<alignment::value_type::string_range>::const_iterator class_e = it->classification.end(); 
        class_it != class_e && organism_tree->ItemHasChildren( insertion_point ) ; ++ class_it  ) {
      wxTreeItemIdValue cookie;
      wxTreeItemId item_child = organism_tree->GetFirstChild( insertion_point, cookie );
      for(; item_child != wxTreeItemId(wxTreeItemIdValue(0)) 
          ; item_child = organism_tree->GetNextChild( insertion_point, cookie ) 
          )
      {
        TreeItemData *t_i = reinterpret_cast<TreeItemData*>(organism_tree->GetItemData( item_child ));
        std::string f_str( convert_back( t_i->get_lineage().ToAscii() ) );
        if( equals( f_str ,  *class_it ) )
        {
          t_i->add_seq();
          wxString n_text( convert(f_str));
          n_text << _T(" [") << t_i->get_num_seq() << _T("]");
          organism_tree->SetItemText( item_child, n_text );
          break;
        }
      }

      if( item_child == wxTreeItemId(wxTreeItemIdValue(0)) )  {
        wxString group( convert( std::string(class_it->first, class_it->second )).c_str() ); // group === lineage
        wxString item_name = group;
        item_name << _T(" [1]");
        insertion_point = organism_tree->AppendItem( insertion_point, item_name, -1, -1, new TreeItemData( group, -1, group ) );
      }
      else 
        insertion_point = item_child;
    }

    for( ; class_it != it->classification.end(); ++class_it ) {  // if not already in? .. guess .. guess omg!
      wxString group( convert( std::string(class_it->first, class_it->second)).c_str() ); // group === lineage
      wxString lin = group;
      if( first_elem ) 
        lin << _T(" [1]");
      insertion_point = organism_tree->AppendItem( insertion_point, lin, -1, -1, new TreeItemData( group, -1, group ) );
      organism_tree->Expand( insertion_point );
    }
    ostringstream ida; ida << std::string( it->id.first, it->id.second ) ; 
    insertion_point = organism_tree->AppendItem( 
        insertion_point
        , wxString(convert( ida.str()).c_str())
        , -1
        , -1
        , new TreeItemData( wxString( convert( ida.str() ).c_str()), sequence_index, wxString( convert(ida.str() ).c_str())) 
        );
  }
  organism_tree->Expand( organism_item );
}


void ProfileDialog::OnItemRightClick( wxTreeEvent & event )
{
  ShowMenu( event.GetItem(), event.GetPoint() );
}

void ProfileDialog::ShowMenu( wxTreeItemId id, const wxPoint & pt )
{
  wxString title;

  if ( id.IsOk() )
  {
    title << wxT( "Menu for " ) << profiles_tree->GetItemText( id );
  }
  else
  {
    title = wxT( "Menu for no particular item" );
  }

  wxMenu menu( title );
  if( id.IsOk() ){
    TreeItemData * item = reinterpret_cast<TreeItemData *>(organism_tree->GetItemData( id  ) );
    if( item ) {
      size_t num_seq = item->get_num_seq();

      wxString test( _T("Number of sequences: "));
      test << num_seq;
      menu.Append( GroupRename, test );
    }
  }
  wxPoint testPt = GetPosition();

  PopupMenu( & menu, testPt );

}

