/***************************************************************************
 *   Copyright (C) 2005 by Andreas Pokorny                                 *
 *   andreas.pokorny@biozentrum.uni-wuerzburg.de                           *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute * 
 *   it and/or modify it under the terms of the GNU General Public License * 
 *   as published by the Free Software Foundation; either version 2 of the * 
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef PROFDIST_PROFILE_CHOOSER_H_INCLUDED
#define PROFDIST_PROFILE_CHOOSER_H_INCLUDED

#include <string>
#include <vector>
#include <map>
#include <list>
#include <wx/treectrl.h> 
#include <wx/bmpbuttn.h>
#include <wx/dialog.h> 
#include <wx/panel.h> 
#include <wx/splitter.h> 
#include <wx/checkbox.h> 
#include "parser.h"
#include "tree.h"

class TreeItemData : public wxTreeItemData
{
  public:
    TreeItemData();
    TreeItemData( wxString const& lineage, std::size_t seq, wxString const& item_text );

    inline wxString const& get_lineage() const  {return lineage; }
    inline std::size_t get_seq() const { return seq; }
    inline void set_num_seq(std::size_t s ) { num_seq = s; }
    inline std::size_t get_num_seq() const {  return num_seq; }
    inline wxString const& get_text() const { return item_text; }
    void add_seq() { ++num_seq; }

  private:
    wxString lineage, item_text;
    std::size_t seq, num_seq;
};


class ProfileDialog : public wxDialog
{
  public:
    enum ButtonIds {
      SubmitProfile 
    , Ok
    , Cancel
    , Help
    , Organism
    , Profiles
    , ResetData 
    , GroupRename 

    };
    typedef std::list<std::list<std::size_t> > profile_list;
    typedef std::list<std::list<std::string> > name_list;
    enum ProfileFlag { MultiSelect, SingleSelect };
    ProfileDialog( wxWindow * parent, int id, wxString const & title, ProfileFlag flags
        , profdist::alignment const& sequences
        , wxPoint const & pos = wxDefaultPosition
        , wxSize const & size = wxDefaultSize
        , long style = wxDEFAULT_DIALOG_STYLE );


    ProfileDialog( wxWindow * parent, int id, wxString const & title, ProfileFlag flags
        , wxPoint const & pos = wxDefaultPosition
        , wxSize const & size = wxDefaultSize
        , long style = wxDEFAULT_DIALOG_STYLE );



    void get_profiles( profile_list & profiles
        , std::vector<std::string> & profile_names
        , name_list & sequence_names );
    void on_submit_profile( wxCommandEvent & WXUNUSED( event ) );
    void on_ok( wxCommandEvent & WXUNUSED( event ) );
    void on_cancel( wxCommandEvent & WXUNUSED( event ) );
    void on_help( wxCommandEvent & WXUNUSED( event ) );
    void on_reset( wxCommandEvent & WXUNUSED( event ) );
    void createLeftTree( std::map < wxString, int > & mapGroups );

    void create_tree( profdist::alignment const& sequences );
//    void create_profile_subtree( wxTreeItemId & organism_item, wxTreeItemId & profile_item );
    void deleteEmptyParent( wxTreeItemId & item );

    void OnItemRightClick( wxTreeEvent & event );
    void ShowMenu( wxTreeItemId id, const wxPoint & pt );

  private:
    void construct_ui();
    void set_properties();
    void do_layout();
    void get_sub_profiles( wxTreeItemId parent, std::list<size_t>  & profile, std::list<std::string> & seq_name );
    void left_profiles( wxTreeItemId parent, profile_list & profiles, std::vector<std::string> & profile_names,  name_list & sequence_names );
    void submit_subtree( wxTreeItemId & org_item, wxTreeItemId & prof_item );
    bool is_child( wxTreeItemId const& parent, wxTreeItemId const& child );
    std::size_t normalize_array(wxArrayTreeItemIds & ids);
    void reduce_seqs(wxArrayTreeItemIds & ids);

    DECLARE_EVENT_TABLE();

  protected :
    // begin wxGlade: MyDialogGroup::attributes
    wxTreeCtrl* organism_tree;
    wxBitmapButton* submit_profile_button;
    wxPanel* panel_1;
    wxPanel* window_1_pane_1;
    wxTreeCtrl* profiles_tree;
    wxPanel* panel_2;
    wxSplitterWindow* window_1;
    wxCheckBox* chkGroupLeft;
    wxButton *ok_button, *cancel_button, *help_button, *reset_button;

    ProfileFlag flags;
    std::size_t num_profiles;


};

#endif

