/***************************************************************************
 *   Copyright (C) 2005 by Andreas Pokorny                                 *
 *   andreas.pokorny@biozentrum.uni-wuerzburg.de                           *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute * 
 *   it and/or modify it under the terms of the GNU General Public License * 
 *   as published by the Free Software Foundation; either version 2 of the * 
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <boost/algorithm/string.hpp>
#include <boost/algorithm/string/trim.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/lexical_cast.hpp>
#include <wx/wx.h>
#include "icons/arrowright.xpm"
#include "traits_chooser.h"

using namespace boost::algorithm;
using namespace std;


BEGIN_EVENT_TABLE( TraitsDialog, wxDialog ) 
//  EVT_BUTTON( TraitsDialog::TreatAsProteine, TraitsDialog::on_proteine)
//  EVT_BUTTON( TraitsDialog::TreatAsRna, TraitsDialog::on_rna)
	EVT_BUTTON( TraitsDialog::ButtonOk, TraitsDialog::onButtonOk )
	EVT_BUTTON( TraitsDialog::ButtonCancel, TraitsDialog::onButtonCancel )
END_EVENT_TABLE() // end


TraitsDialog::TraitsDialog( wxWindow * parent, int id, 
    wxPoint const& pos, 
    wxSize const & size, long style ) :
  wxDialog( parent, id, _T("Define the type of input")
      , pos
      , size
      , wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxSYSTEM_MENU | wxMAXIMIZE_BOX | wxMINIMIZE_BOX )
{
  construct_ui();
}

void TraitsDialog::construct_ui() {
  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL);
  //rna_button = new wxButton(panel, TreatAsRna, _("RNA"));
  //proteine_button = new wxButton(panel, TreatAsProteine, _("Proteine"));
  const wxString choices[2] = {
    _T("Treat as RNA"),
	_T("Treat as Proteine")
  };
  radiobox = new wxRadioBox(this, -1, _T("Select input type:"), wxDefaultPosition, wxDefaultSize, 2, choices);
  ok_button = new wxButton(this, ButtonOk, _T("Ok"));
  cancel_button = new wxButton(this, ButtonCancel, _T("Cancel"));
  set_properties();
  do_layout();
}


void TraitsDialog::set_properties()
{
  // begin wxGlade: TraitsDialog::set_properties
  //rna_button->SetFocus();
  //rna_button->SetDefault();
  ok_button->SetFocus();
  ok_button->SetDefault();
  // end wxGlade
}


void TraitsDialog::do_layout()
{
  // begin wxGlade: TraitsDialog::do_layout
//   wxBoxSizer* sizer= new wxBoxSizer(wxHORIZONTAL);
//   sizer->Add(rna_button, 1, wxEXPAND, 12);
//   sizer->Add(5, 5, 0, 0, 12);
//   sizer->Add(proteine_button, 1, wxEXPAND, 12);
  
  wxBoxSizer* sizer1 = new wxBoxSizer(wxHORIZONTAL);
  sizer1->Add(radiobox, wxSizerFlags(0).Center().Border(wxALL, 10));
  
  wxBoxSizer* sizer2 = new wxBoxSizer(wxHORIZONTAL);
  sizer2->Add(ok_button, wxSizerFlags(0).Center().Border(wxALL, 5));
  sizer2->Add(cancel_button, wxSizerFlags(0).Center().Border(wxALL, 5));
  
  wxBoxSizer* sizerAll = new wxBoxSizer(wxVERTICAL);
  sizerAll->Add(sizer1, wxSizerFlags(0).Center().Border(wxALL, 5));
  sizerAll->Add(sizer2, wxSizerFlags(0).Center().Border(wxALL, 5));
  
  sizerAll->Fit(this);
  sizerAll->SetSizeHints(this);
  
  SetAutoLayout(true);
  SetSizer(sizerAll);
  Layout();
  Centre();
  // end wxGlade
}

// void TraitsDialog::on_rna( wxCommandEvent & WXUNUSED( event ) )
// {
//   EndModal( TreatAsRna );
// }
// 
// void TraitsDialog::on_proteine( wxCommandEvent & WXUNUSED( event ) )
// {
//   EndModal( TreatAsProteine  );
// }

void TraitsDialog::onButtonOk( wxCommandEvent & WXUNUSED( event ) )
{
	EndModal(static_cast<Choice>(radiobox->GetSelection()));
}

void TraitsDialog::onButtonCancel( wxCommandEvent & WXUNUSED( event ) )
{
	EndModal(Cancel);
}





