/***************************************************************************
 *   Copyright (C) 2005 by Joachim Friedrich                               *
 *   joachim.friedrich@biozentrum.uni-wuerzburg.de                         *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute * 
 *   it and/or modify it under the terms of the GNU General Public License * 
 *   as published by the Free Software Foundation; either version 2 of the * 
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ProfDist.h"

//---------------------------------------------------------------------------
//
//  name          : onMnuPref
//
//  description   : Load a text file in Input Window from Bootstrap
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::getPreferences( long & numberBoot, profdist::CorrectionModel& evoModel, profdist::ExpMType & expm, profdist::FZero & fMax, wxString & fileRateMatrix,
        long & pnjMethod, long & pnjBootstrapThreshold, long & pnjIdentityThreshold, long & pnjShowOC, long & treeViewType,
        wxString & fileTreeViewer1, wxString & fileTreeViewer2 )
{
  txtPrefNumberBoot->GetValue().ToLong( & numberBoot );
  evoModel = static_cast<profdist::CorrectionModel>(rdoPrefEvolution->GetSelection());
  //expm = static_cast<profdist::ExpMType>(rdoPrefMatrixExpm->GetSelection());
  expm = profdist::Eigenvalue;
  //expm = profdist::Taylor;
  //fMax = static_cast<profdist::FZero>(rdoPrefMaximum->GetSelection());
  fMax = (rdoPrefMaximum->GetSelection() == 0 ? profdist::NewtonMethod : profdist::Robust);
  fileRateMatrix = txtPrefRatematrix->GetValue();
  pnjMethod = rdoPrefPnjType->GetSelection();

  txtPrefProfBootValue->GetValue().ToLong( & pnjBootstrapThreshold );
  txtPrefProfPercentIdent->GetValue().ToLong( & pnjIdentityThreshold );

  if ( chkPrefShowOC->IsChecked() )
    pnjShowOC = 1;
  else
    pnjShowOC = 0;

  treeViewType = mPrefTreeType;

  fileTreeViewer1 = txtPrefTreeViewer1->GetValue();
  fileTreeViewer2 = txtPrefTreeViewer2->GetValue();
}


//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefOpenTreeViewer1( wxCommandEvent & WXUNUSED( event ) )
{
  wxFileDialog dialog( this, _T( "Open Dialog" ), _T( "" ), _T( "" ),
#ifdef __WXMSW__
      _T( "Executables (*.exe)|*.exe" )
#else
      _T( "All files (*)|*" )
#endif
      );

  if ( dialog.ShowModal() == wxID_OK )
    txtPrefTreeViewer1->SetValue( dialog.GetPath() );
}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefOpenTreeViewer2( wxCommandEvent & WXUNUSED( event ) )
{
  wxFileDialog dialog( this, _T( "Open Dialog" ), _T( "" ), _T( "" ),
#ifdef __WXMSW__
      _T( "Executables (*.exe)|*.exe" )
#else
      _T( "All files (*)|*" )
#endif
      );

  if ( dialog.ShowModal() == wxID_OK )
    txtPrefTreeViewer2->SetValue( dialog.GetPath() );
}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefOpenRatematrix( wxCommandEvent & WXUNUSED( event ) )
{
  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ),
#ifdef __WXMOTIF__
      _T( "All files (*.*)|*.*" )
#else
      _T( "All files (*.*)|*.*" )
#endif
      );

  if ( dialog.ShowModal() == wxID_OK )
    txtPrefRatematrix->SetValue( dialog.GetPath() );

}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefOk( wxCommandEvent & WXUNUSED( event ) )
{
  ( * this ).EndModal( idBtnPrefOk );
}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefCancel( wxCommandEvent & WXUNUSED( event ) )
{
  ( * this ).EndModal( idBtnPrefCancel );
}

void MyDialogPref::onRdoPrefTree1( wxCommandEvent & WXUNUSED( event ) )
{
  mPrefTreeType = 0;
}

void MyDialogPref::onRdoPrefTree2( wxCommandEvent & WXUNUSED( event ) )
{
  mPrefTreeType = 1;
}

void MyDialogPref::onRdoPrefHypertree( wxCommandEvent & WXUNUSED( event ) )
{
  mPrefTreeType = 2;
}

void MyDialogPref::onRdoPrefNone( wxCommandEvent & WXUNUSED( event ) )
{
  mPrefTreeType = 3;
}

void MyDialogPref::onRdoPrefEvolution( wxCommandEvent & WXUNUSED( event ) )
{
  if ( rdoPrefEvolution->GetSelection() < 2 )
  {
    //rdoPrefMatrixExpm->Enable( false );
    rdoPrefMaximum->Enable( false );
  }
  else
  {
    //rdoPrefMatrixExpm->Enable( true );
    rdoPrefMaximum->Enable( true );
  }
}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefHelp( wxCommandEvent & WXUNUSED( event ) )
{
  help.Display( wxT( "ProfDist Helpfile" ) );
}

//---------------------------------------------------------------------------
//
//  name          : onBtnPnjOpenFile
//
//  description   : Load a text file in Input Window from Profile Neighbor Joining
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::onBtnPrefExpert( wxCommandEvent & WXUNUSED( event ) )
{
  if ( rdoPrefMaximum->IsEnabled() )
  {
    //rdoPrefMatrixExpm->Enable( false );
    rdoPrefMaximum->Enable( false );
  }
  else
  {
    //rdoPrefMatrixExpm->Enable( true);
    rdoPrefMaximum->Enable( true );
  }
}

//---------------------------------------------------------------------------
//
//  name          : onMnuPref
//
//  description   : Load a text file in Input Window from Bootstrap
//
//  input         : *this
//
//  return value  : void
//
//---------------------------------------------------------------------------
void MyDialogPref::setPreferences( long & numberBoot, profdist::CorrectionModel const& evoModel, profdist::ExpMType const& expm, profdist::FZero const& fMax, wxString & fileRateMatrix,
        long & pnjMethod, long & pnjBootstrapThreshold, long & pnjIdentityThreshold, long & pnjShowOC, long & treeViewType,
        wxString & fileTreeViewer1, wxString & fileTreeViewer2 )
{
  wxString temp;
  temp << numberBoot;
  txtPrefNumberBoot->SetValue( temp );
  temp.Clear();

  rdoPrefEvolution->SetSelection( evoModel );

  if ( evoModel < 2 )
  {
    //rdoPrefMatrixExpm->Enable( false );
    rdoPrefMaximum->Enable( false );
  }
  else
  {
    //rdoPrefMatrixExpm->Enable( true );
    rdoPrefMaximum->Enable( true );
  }

  //rdoPrefMatrixExpm->SetSelection( expm );
  rdoPrefMaximum->SetSelection( (fMax == profdist::NewtonMethod ? 0 : 1) );
  txtPrefRatematrix->SetValue( fileRateMatrix );
  rdoPrefPnjType->SetSelection( pnjMethod );

  temp << pnjBootstrapThreshold;
  txtPrefProfBootValue->SetValue( temp );
  temp.Clear();

  temp << pnjIdentityThreshold;
  txtPrefProfPercentIdent->SetValue( temp );
  temp.Clear();

  if ( pnjShowOC == 1 )
    chkPrefShowOC->SetValue( true );
  else
    chkPrefShowOC->SetValue( false );

  if ( treeViewType == 0 )
    rdoPrefTree1->SetValue( true );
  else if ( int( treeViewType ) == 1 )
    rdoPrefTree2->SetValue( true );
  else
    rdoPrefNoneTree->SetValue( true );

  mPrefTreeType = treeViewType;
  txtPrefTreeViewer1->SetValue( fileTreeViewer1 );
  txtPrefTreeViewer2->SetValue( fileTreeViewer2 );
}
