/***************************************************************************
 *   Copyright (C) 2005 by Joachim Friedrich                               *
 *   joachim.friedrich@biozentrum.uni-wuerzburg.de                         *
 *                                                                         *
 *   This file is part of profdist and cbcanalyzer                         *
 *                                                                         *
 *   Both profdist and cbcanalyzer are free software; you can redistribute *
 *   it and/or modify it under the terms of the GNU General Public License *
 *   as published by the Free Software Foundation; either version 2 of the *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   Profdist and cbcanalyzer are distributed in the hope that it will be  *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iterator>
#include <ctime>
#include <boost/spirit/core.hpp>
#include <boost/spirit/core/primitives/numerics.hpp>
#include <boost/spirit/core/primitives/primitives.hpp>
#include <boost/spirit/core/composite/kleene_star.hpp>
#include <boost/spirit/actor/push_back_actor.hpp>
#include <boost/spirit/utility/lists.hpp>
#include <boost/lexical_cast.hpp>
#include "ProfDist.h"
#include "temp_files.h"
#include "parser.h"
#include "aligncode.h"
#include "bionj_clean.h"
#include "bootstrap.h"
#include "converter.h"
#include "distance.h"
#include "consense.h"
#include "progress_dialog.h"
#include "profile_chooser.h"
#include "traits_chooser.h"
#include "spirit_helper.h"
#include "wx_streambuf.h"
#ifdef DARWIN_SYSTEM
#include <Carbon/Carbon.h>
#endif

//#define DANIEL_CODE
using namespace profdist;


//---------------------------------------------------------------------------

void MyFrame::openFileBoot()
{
  wxString strFileFormat;
  strFileFormat = setOpenFileFormatAlignmentBoot();

  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ), strFileFormat );
  dialog.SetFilterIndex( rdoBootInput->GetSelection() );
  if ( dialog.ShowModal() == wxID_OK )
  {
    ::wxBeginBusyCursor();
    if ( chkBootInput->IsChecked() )
      txtBootInput->LoadFile( dialog.GetPath().c_str() );
    rdoBootInput->SetSelection( dialog.GetFilterIndex() );
    mBootFilePath = dialog.GetPath();
    ::wxEndBusyCursor();
  }
}

void MyFrame::openFileNj()
{
  wxString strFileFormat;

  strFileFormat = _T("Matrix files (*.mat)|*.mat|Text files (*.txt)|*.txt|All files (*)|*");

  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ), strFileFormat );

  if ( dialog.ShowModal() == wxID_OK )
  {
    ::wxBeginBusyCursor();
    if ( chkNjInput->IsChecked() )
      txtNjInput->LoadFile( dialog.GetPath().c_str() );

    mNjFilePath = dialog.GetPath();
    rdoNjInput->SetSelection( dialog.GetFilterIndex() );
    ::wxEndBusyCursor();
  }
}

void MyFrame::openFileCons()
{
  wxString strFileFormat;

  strFileFormat = _T("Matrix files (*.tre)|*.tre|All files (*)|*");

  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ), strFileFormat );

  if ( dialog.ShowModal() == wxID_OK )
  {
    ::wxBeginBusyCursor();
    txtConsInput->LoadFile( dialog.GetPath().c_str() );

    mConsFilePath = dialog.GetPath();
    rdoConsInput->SetSelection( 0 );
    ::wxEndBusyCursor();
  }
}


void MyFrame::openFilePnj()
{
  wxString strFileFormat;
  strFileFormat = setOpenFileFormatAlignment();

  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ), strFileFormat ); //, wxMULTIPLE );
  dialog.SetFilterIndex( rdoPnjInput->GetSelection() );

  /*if ( dialog.ShowModal() == wxID_OK ) { wxArrayString fileList; dialog.GetPaths( fileList ); txtPnjInput->Clear();
    wxString temp; for ( int i = 0; i < fileList.GetCount(); ++i ) { txtPnjInput->LoadFile( fileList.Item( i ).c_str() );
    temp += txtPnjInput->GetValue() + "\n"; } txtPnjInput->SetValue( temp ); }*/
  if ( dialog.ShowModal() == wxID_OK )
  {
    ::wxBeginBusyCursor();
    if ( chkPnjInput->IsChecked() )
      txtPnjInput->LoadFile( dialog.GetPath().c_str() );

    rdoPnjInput->SetSelection( dialog.GetFilterIndex() );
    mPnjFilePath = dialog.GetPath();
    ::wxEndBusyCursor();
  }
}



bool MyFrame::create_profile_dialog( profdist::profile_set & profiles )
{
  MyDialogProfile dialog( this, -1, _T("") );

  while( true ){
    switch( dialog.ShowModal() )
    {
      case idBtnProfileOk:
        {
          std::string profile_string = convert_back( dialog.getProfile().ToAscii() );
          using boost::spirit::parse;
          using boost::spirit::parse_info;
          using boost::spirit::uint_p;
          using boost::spirit::space_p;
          using boost::spirit::ch_p;

          parse_info<> result = parse( profile_string.c_str() , 
              uint_p[set_insert_a(profiles)] 
              >> *(
                ( ch_p('/') |  ',' |  ';' | '.' )
                >> uint_p[set_insert_a(profiles)] 
                ) 
              >> *
              ( ch_p('/') |  ',' |  ';' | '.' )
              ,space_p
              );
          return true;
        }
  case idBtnProfileHelp: 
#ifdef DARWIN_SYSTEM
        help.Display( wxT( "ProfDist Helpfile" ) );
#endif
        break;
      default: return false;
    }
  }
}

size_t MyFrame::is_proteine() 
{
  TraitsDialog d(this,-1);
  while(true) {
    switch(d.ShowModal()) {
      case TraitsDialog::TreatAsRna:std::cout << "Treating as RNA" << std::endl; return 0;
      case TraitsDialog::TreatAsProteine:std::cout << "Treating as Proteine" << std::endl;return 1;
	  case TraitsDialog::Cancel: return TraitsDialog::Cancel;
    }
  }
}

bool MyFrame::create_group_dialog( profdist::alignment const& sequences, list<list<size_t> > & profiles,
    vector<string> & prof_names, list<list<string> >  & names )
{
  ProfileDialog dialog( this, -1, _T("Select Profiles"), ProfileDialog::MultiSelect, sequences );

  while(true) {
    switch ( dialog.ShowModal() ) {
      case ProfileDialog::Ok:  
        dialog.get_profiles( profiles, prof_names, names );
        if( prof_names.size() >= 3 ) return true;
        dialog.SetTitle(_T("Define more than three Profiles"));
        break;
      case ProfileDialog::Cancel: return false;
      case ProfileDialog::Help: 
#ifdef DARWIN_SYSTEM
                                  help.Display( wxT( "ProfDist Helpfile" ) );
#endif
                                  break;
      case ProfileDialog::ResetData: dialog.create_tree(sequences);break;
      default: return false;
    }
  }
}

void MyFrame::savePnj()
{
  wxString strFileFormat = _T("Newick (*.tre)|*.tre|All file (*)|*|");

  wxFileDialog dialog( this, _T( "Save File Dialog" ), _T( "" ), _T( "" ), strFileFormat, wxSAVE | wxOVERWRITE_PROMPT );

  if ( dialog.ShowModal() == wxID_OK )
    txtPnjOutput->SaveFile( dialog.GetPath() );
}

void MyFrame::onChkBootInput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkBootInput->IsChecked() )
  {
    txtBootInput->Enable( true );
    rdoBootInput->Enable( true );
  }
  else
  {
    txtBootInput->Clear();
    txtBootInput->Enable( false );
    rdoBootInput->Enable( false );
  }
}

void MyFrame::onChkNjInput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkNjInput->IsChecked() )
  {
    txtNjInput->Enable( true );
    rdoNjInput->Enable( true );
  }
  else
  {
    txtNjInput->Clear();
    txtNjInput->Enable( false );
    rdoNjInput->Enable( false );
  }
}

void MyFrame::onChkDistInput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkDistInput->IsChecked() )
  {
    txtDistInput->Enable( true );
    rdoDistInput->Enable( true );
  }
  else
  {
    txtDistInput->Clear();
    txtDistInput->Enable( false );
    rdoDistInput->Enable( false );
  }
}


void MyFrame::onChkPnjInput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkPnjInput->IsChecked() )
  {
    txtPnjInput->Enable( true );
    rdoPnjInput->Enable( true );
  }
  else
  {
    txtPnjInput->Clear();
    txtPnjInput->Enable( false );
    rdoPnjInput->Enable( false );
  }
}

void MyFrame::onChkBootOutput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkBootOutput->IsChecked() )
  {
    txtBootOutput->Enable( true );
    rdoBootOutput->Enable( true );
  }
  else
  {
    txtBootOutput->Clear();
    txtBootOutput->Enable( false );
    rdoBootOutput->Enable( false );
  }
}

void MyFrame::onChkDistOutput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkDistOutput->IsChecked() )
  {
    txtDistOutput->Enable( true );
    rdoDistOutput->Enable( true );
  }
  else
  {
    txtDistOutput->Clear();
    txtDistOutput->Enable( false );
    rdoDistOutput->Enable( false );
  }
}

void MyFrame::onChkNjOutput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkNjOutput->IsChecked() )
  {
    txtNjOutput->Enable( true );
    rdoNjOutput->Enable( true );
  }
  else
  {
    txtNjOutput->Clear();
    txtNjOutput->Enable( false );
    rdoNjOutput->Enable( false );
  }
}

void MyFrame::onChkPnjOutput( wxCommandEvent & WXUNUSED( event ) )
{
  if ( chkPnjOutput->IsChecked() )
  {
    txtPnjOutput->Enable( true );
    rdoPnjOutput->Enable( true );
  }
  else
  {
    txtPnjOutput->Clear();
    txtPnjOutput->Enable( false );
    rdoPnjOutput->Enable( false );
  }
}

void MyFrame::onMnuOpen( wxCommandEvent & WXUNUSED( event ) )
{
  int idxCurrent = ntbAll->GetSelection();

  switch ( idxCurrent )
  {
    case 0: // Bootstrap
      openFileBoot();
      break;
    case 1: // Distance
      openFileDist();
      break;
    case 2: // Nj
      openFileNj();
      break;
    case 3: // Consense
      openFileCons();
      break;
    case 4: // Profile Neighbor Joining
      openFilePnj();
      break;
  }
}


void MyFrame::onMnuSave( wxCommandEvent & WXUNUSED( event ) )
{
  int new_selection = ntbAll->GetSelection();
  wxString ntbName = ntbAll->GetPageText( new_selection );

  int idxCurrent = ntbAll->GetSelection();

  switch ( idxCurrent )
  {
    case 0: // Bootstrap
      {
        if ( txtBootOutput->GetLastPosition() > 0 )
          saveBoot();
      }
      break;
    case 1: // Distance
      {
        if ( txtDistOutput->GetLastPosition() > 0 )
          saveDist();
      }
      break;
    case 2: // Nj
      {
        if ( txtNjOutput->GetLastPosition() > 0 )
          saveNj();
      }
      break;
    case 3: // Consense
      {
        if ( txtConsOutput->GetLastPosition() > 0 )
          saveCons();
      }
      break;
    case 4: // Profile Neighbor Joining
      {
        if ( txtPnjOutput->GetLastPosition() > 0 )
          savePnj();
      }
      break;
  }

}

void MyFrame::onMnuSelectAll( wxCommandEvent & WXUNUSED( event ) )
{
  wxTextCtrl * tempTextCtrl = getFocusedText();

  if( tempTextCtrl  )
    tempTextCtrl->SetSelection( -1, -1 );
}


void MyFrame::onMnuPref( wxCommandEvent & WXUNUSED( event ) )
{
  MyDialogPref dialog( this, -1, _T("") );

  dialog.setPreferences( mNumberBoot, mEvoModel, mExpm, mFMax, mFileRateMatrix, mPnjMethod, mPnjBootstrapThreshold,
      mPnjIdentityThreshold, mPnjShowOC, mTreeViewType, mFileTreeViewer1, mFileTreeViewer2 );

  if ( dialog.ShowModal() == idBtnPrefOk )
  {
    dialog.getPreferences( mNumberBoot, mEvoModel, mExpm, mFMax, mFileRateMatrix, mPnjMethod, mPnjBootstrapThreshold,
        mPnjIdentityThreshold, mPnjShowOC, mTreeViewType, mFileTreeViewer1, mFileTreeViewer2 );

    saveXmlFile();
  }
}


void MyFrame::onMnuStart( wxCommandEvent & WXUNUSED( event ) )
{
  ::wxBeginBusyCursor();
  try {
    int new_selection = ntbAll->GetSelection();
    wxString ntbName = ntbAll->GetPageText( new_selection );

    int idxCurrent = ntbAll->GetSelection();

    switch ( idxCurrent )
    {
      case 0: // Bootstrap
        if ( ( txtBootInput->GetLastPosition() > 0 && chkBootInput->IsChecked() )
            || ( !mBootFilePath.IsEmpty() && !chkBootInput->IsChecked() ) ) {
          startBoot();
          flags[0] = 1;
        }
        break;
      case 1: // Distance
        if ( ( txtDistInput->GetLastPosition() > 0 && chkDistInput->IsChecked() )
            || ( !mDistFilePath.IsEmpty() && !chkDistInput->IsChecked() ) ){
          startDist();
          flags[0] = 0;
          flags[1] = 1;
        }
        break;
      case 2: // Nj
        if ( ( txtNjInput->GetLastPosition() > 0 && chkNjInput->IsChecked() )
            || ( !mNjFilePath.IsEmpty() && !chkNjInput->IsChecked() ) ) {
          startNj();
          flags[1] = 0;
          flags[2] = 1;
        }
        break;
      case 3: // Consense
        if ( txtConsInput->GetLastPosition() > 0  ) {
          startCons();
          flags[2] = 0;
          flags[3] = 1;
        }
        break;
      case 4: // Profile Neighbor Joining
        if ( ( txtPnjInput->GetLastPosition() > 0 && chkPnjInput->IsChecked() )
            || ( !mPnjFilePath.IsEmpty() && !chkPnjInput->IsChecked() ) )
          startPnj();
        break;
    }
  }catch( error const & e ){
    error_output( this, e );
  }catch ( std::exception const & e ){
    error_output( this, e );
  }catch( ... ){
    error_output( this, "Unknown exception, caught");
  }
  ::wxEndBusyCursor();
}



void MyFrame::onMnuTreeView( wxCommandEvent & WXUNUSED( event ) )
{
  int new_selection = ntbAll->GetSelection();
  wxString ntbName = ntbAll->GetPageText( new_selection );

  int idxCurrent = ntbAll->GetSelection();

  switch ( idxCurrent )
  {
    case 2: // Nj
      treeViewNj();
      break;
    case 3: // Consense
      treeViewCons();
      break;
    case 4: // Profile Neighbor Joining
      treeViewPnj();
      break;
  }
}

void MyFrame::onMnuGamma( wxCommandEvent & WXUNUSED( event ) )
{
  int new_selection = ntbAll->GetSelection();
  wxString ntbName = ntbAll->GetPageText( new_selection );

  int idxCurrent = ntbAll->GetSelection();

  switch ( idxCurrent )
  {
    case 2: // Nj
      treeViewNj();
      break;
    case 3: // Consense
      treeViewCons();
      break;
    case 4: // Profile Neighbor Joining
      treeViewPnj();
      break;
  }
}

void MyFrame::onMnuHelp( wxCommandEvent & WXUNUSED( event ) )
{
  help.Display( wxT( "ProfDist Helpfile" ) );
}

void MyFrame::onMnuAbout( wxCommandEvent & WXUNUSED( event ) )
{
  MyDialogAbout dialog( this, -1, _T("") );

  dialog.ShowModal();
}


void MyFrame::onMnuExit( wxCommandEvent & WXUNUSED( event ) )
{
  Close( true );
}

void MyFrame::onCloseEvent( wxCloseEvent & event )
{
  // Close the help frame; this will cause the config data to
  // get written.
  if ( help.GetFrame() ) // returns NULL if no help frame active
    help.GetFrame()->Close( TRUE );
  // now we can safely delete the config pointer
  event.Skip();
  delete wxConfig::Set( NULL );

  saveXmlFile();
  Destroy();
  TempFile::cleanup();
}

void MyFrame::onMnuUndo( wxCommandEvent & WXUNUSED( event ) )
{
  wxTextCtrl * tempTextCtrl = getFocusedText();

  if( tempTextCtrl && tempTextCtrl->CanUndo() )
    tempTextCtrl->Undo();
}

void MyFrame::onMnuCut( wxCommandEvent & WXUNUSED( event ) )
{
  wxTextCtrl * tempTextCtrl = getFocusedText();
  if( tempTextCtrl && tempTextCtrl->CanCut() )
    tempTextCtrl->Cut();
}

void MyFrame::onMnuCopy( wxCommandEvent & WXUNUSED( event ) )
{
  wxTextCtrl * tempTextCtrl = getFocusedText();
  if( tempTextCtrl && tempTextCtrl->CanCopy() )
    tempTextCtrl->Copy();
}

void MyFrame::onMnuPaste( wxCommandEvent & WXUNUSED( event ) )
{
  wxTextCtrl * tempTextCtrl = getFocusedText();
  if( tempTextCtrl && tempTextCtrl->CanPaste() )
    tempTextCtrl->Paste();
}


void MyFrame::copy_text_window(wxTextCtrl * source, wxTextCtrl * destination)  const
{
  destination->SetValue( source->GetValue() );
}

void MyFrame::onNotebook( wxNotebookEvent & event )
{
  switch( event.GetSelection() ){
    case 1: // Distance 
      if( flags[0] ) {
        wxMessageDialog dialog( this, _T("Bootstrap has a new output.\nWould you like to copy the data?"), _T("Question"),
            wxYES_NO | wxICON_QUESTION );
        if ( dialog.ShowModal() == wxID_YES )
        {
          copy_text_window( txtBootOutput, txtDistInput );
          rdoDistInput->SetSelection( 0 );
          chkDistMultiple->SetValue( true );
          flags[0] = 0;
        }
      }
      break;
    case 2: // nj
      if( flags[1] ) {
        wxMessageDialog dialog( this, _T("Distance has a new output.\nWould you like to copy the data?"), _T("Question"),
            wxYES_NO | wxICON_QUESTION );
        if ( dialog.ShowModal() == wxID_YES )
        {
          copy_text_window( txtDistOutput, txtNjInput );
          rdoDistInput->SetSelection( rdoBootOutput->GetSelection() );
          if ( chkDistMultiple->GetValue() == true )
            chkNjMultiple->SetValue( true );
          else
            chkNjMultiple->SetValue( false );
          flags[1] = 0;
        }
      }
      break;
    case 3: // consense
      if( flags[2] ) {
        wxMessageDialog dialog( this, _T("Distance has a new output.\nWould you like to copy the data?"), _T("Question"),
            wxYES_NO | wxICON_QUESTION );
        if ( dialog.ShowModal() == wxID_YES ) {
          copy_text_window( txtNjOutput, txtConsInput );
          flags[2] = 0;
        }
      }
      break;
    default:break;
  }
}



void MyFrame::saveXmlFile()
{
  wxConfigBase* config = wxConfigBase::Get( true );
  config->Write(_T("/profdist/bootstrap_number"), mNumberBoot );
  config->Write(_T("/profdist/evolutionary_model"), mEvoModel);
  config->Write(_T("/profdist/matrix_exponential"), mExpm );
  config->Write(_T("/profdist/function_maximation"), mFMax );
  config->Write(_T("/profdist/rate_matrix"), mFileRateMatrix );
  config->Write(_T("/profdist/pnj/method"), mPnjMethod );
  config->Write(_T("/profdist/pnj/boostrap_threshold"), mPnjBootstrapThreshold );
  config->Write(_T("/profdist/pnj/identiy_threshold"), mPnjIdentityThreshold );
  config->Write(_T("/profdist/pnj/show_OC"), mPnjShowOC );
  config->Write(_T("/profdist/treeviewer/view_type"), mTreeViewType );
  config->Write(_T("/profdist/treeviewer/binary_path_1"), mFileTreeViewer1 );
  config->Write(_T("/profdist/treeviewer/binary_path_2"), mFileTreeViewer2 );


  config->Write(_T("/profdist/gui/notebook_selection"), ntbAll->GetSelection() );
  config->Write(_T("/profdist/gui/boot_input"), rdoBootInput->GetSelection() );
  config->Write(_T("/profdist/gui/boot_output"), rdoBootOutput->GetSelection() );
  config->Write(_T("/profdist/gui/dist_input"), rdoDistInput->GetSelection() );
  config->Write(_T("/profdist/gui/dist_output"), rdoDistOutput->GetSelection() );
  config->Write(_T("/profdist/gui/nj_input"), rdoNjInput->GetSelection() );
  config->Write(_T("/profdist/gui/nj_output"), rdoNjOutput->GetSelection() );
  config->Write(_T("/profdist/gui/cons_output"), rdoConsOutput->GetSelection() );
  config->Write(_T("/profdist/gui/pnj_input"), rdoPnjInput->GetSelection() );
  config->Write(_T("/profdist/gui/pnj_output"), rdoPnjOutput->GetSelection() );

  config->Write(_T("/profdist/gui/chk_boot_input"),  chkBootInput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_boot_output"), chkBootOutput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_dist_input"),  chkDistInput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_dist_output"), chkDistOutput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_nj_input"),    chkNjInput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_nj_output"),   chkNjOutput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_pnj_input"),   chkPnjInput->IsChecked() );
  config->Write(_T("/profdist/gui/chk_pnj_output"),  chkPnjOutput->IsChecked() );
}



void MyFrame::setSelection()
{
  ntbAll->SetSelection( mNtbSelection );
  rdoBootInput->SetSelection( mRdoBootInput );
  rdoBootOutput->SetSelection( mRdoBootOutput );
  rdoDistInput->SetSelection( mRdoDistInput );
  rdoDistOutput->SetSelection( mRdoDistOutput );
  rdoNjInput->SetSelection( mRdoNjInput );
  rdoNjOutput->SetSelection( mRdoNjOutput );
  rdoConsOutput->SetSelection( mRdoConsOutput );
  rdoPnjInput->SetSelection( mRdoPnjInput );
  rdoPnjOutput->SetSelection( mRdoPnjOutput );
}

template<typename Traits>
bool MyFrame::startSpecializedBoot()
{
	bool cont = true;
	int ip;
	int iInputFileType, iOutputFileType;
	AlignCode<Traits> alignCode;

//   wxProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ),
//       mNumberBoot, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
	ProgressDialog dialog(_T( "Progress dialog" ), _T( "Duration of step: 1" ),
		mNumberBoot, this, wxPD_CAN_ABORT | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME);

	iInputFileType = rdoBootInput->GetSelection();
	iOutputFileType = rdoBootOutput->GetSelection();
	
	try{
		if ( chkBootInput->IsChecked() )
		{
			TempFile t( _T("bootInput") );
			txtBootInput->SaveFile( t.file_name );
			parse_file( alignCode, convert_back( t.file_name.ToAscii() ), (iInputFileType == 0)?profdist::Fasta:profdist::Embl);
		}
		else
			parse_file( alignCode, convert_back( mBootFilePath.ToAscii() ), (iInputFileType == 0)?profdist::Fasta:profdist::Embl);
		if( alignCode.get_num_sequences() < 2 ) 
			throw runtime_error("not enough sequences for bootstrap found");
	} catch( std::exception const& e ) {
		if( alignCode.get_num_sequences() >= 2 ) {
			std::ostringstream msg;
			msg << "Continue Bootstrap?\n" 
					<< e.what() 
					<< " but " << alignCode.get_num_sequences() 
					<< " Sequences found." << endl; 
			if( wxMessageBox(  convert(msg.str() ), _T( "Continue Bootstrap?" ), // caption
					wxYES_NO | wxICON_QUESTION ) != wxYES )
				return false;
		}
		else {
			error_output( this, e );
			return false;
		}
	} catch( ... ) {
		error_output( this, "Unknown Exception caught");
		return false;
	}

	txtBootOutput->Clear();

	{
		wx_streambuf buf( *txtBootOutput );
		ostream out(&buf);
		if( mNumberBoot )
		{
			for ( int i = 0; i < mNumberBoot; ++i )
			{
				if ( i == mNumberBoot )
					cont = dialog.update( i, "Task finished!" );
				else
					cont = dialog.update( i );

				if ( !cont )
				{
					if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
							 wxYES_NO | wxICON_QUESTION ) == wxYES )
						return false;
					dialog.Resume();
				}
				AlignCode<Traits> alignBootstrap; 

				bootstrap( alignCode, alignBootstrap );
				write_file( alignBootstrap, out, profdist::Fasta );
				out << "\n";
			}
		}
		else 
		{
			write_file( alignCode, out, profdist::Fasta );
			out << "\n";
      
			dialog.update( mNumberBoot, "Task finished!" );
		}

		out.flush();
	}
	move_to_top( txtBootOutput );
	return true;
}


bool MyFrame::startBoot()
{
  int iInputFileType, iOutputFileType;

  iInputFileType = rdoBootInput->GetSelection();
  iOutputFileType = rdoBootOutput->GetSelection();
	
	size_t ip; 
	if( iInputFileType > 2 )
		ip = 2;
	else ip = is_proteine();
	
	switch(ip)
	{
		case 0: return startSpecializedBoot<profdist::rna_traits>();
		case 1: return startSpecializedBoot<profdist::protein_traits>();
		case 2: return startSpecializedBoot<profdist::rna_structure_traits>();
		default: return false;
	}
}



void MyFrame::saveBoot()
{
  wxFileDialog dialog( this, _T( "Save File Dialog" ), _T( "" ), _T( "" ),
      _T( "All files (*)|*" )
      , wxSAVE | wxOVERWRITE_PROMPT );
  //_T("Text files (*.txt)|*.txt|Document files (*.doc)|*.doc"),

  if ( dialog.ShowModal() == wxID_OK )
    txtBootOutput->SaveFile( dialog.GetPath() );
}



void MyFrame::openFileDist()
{
  wxString strFileFormat;
  strFileFormat = setOpenFileFormatAlignment();

  wxFileDialog dialog( this, _T( "Open File Dialog" ), _T( "" ), _T( "" ), strFileFormat );
  dialog.SetFilterIndex( rdoDistInput->GetSelection() );
  if ( dialog.ShowModal() == wxID_OK )
  {
    ::wxBeginBusyCursor();
    if ( chkDistInput->IsChecked() )
      txtDistInput->LoadFile( dialog.GetPath().c_str() );
    rdoDistInput->SetSelection( dialog.GetFilterIndex() );
    mDistFilePath = dialog.GetPath();
    ::wxEndBusyCursor();
  }

}


wxString MyFrame::setOpenFileFormatAlignmentBoot()
{
  wxString strFileFormat;

  strFileFormat = _T("Fasta (*.fas)|*.fas|");
  strFileFormat += _T("Embl (*.embl)|*.embl|");
  strFileFormat += _T("All files (*)|*");

  return strFileFormat;
}
wxString MyFrame::setOpenFileFormatAlignment()
{
  wxString strFileFormat;

  strFileFormat = _T("Fasta (*.fas)|*.fas|");
  strFileFormat += _T("Embl (*.embl)|*.embl|");
//   strFileFormat += _T("Bdb (*)|*|");
//   strFileFormat += _T("Marna (*)|*|");
//   strFileFormat += _T("Forester (*)|*|");
  strFileFormat += _T("All files (*)|*");

  return strFileFormat;
}

wxString MyFrame::setSaveFileFormatAlignment()
{
  wxString strFileFormat;

  strFileFormat = _T("Fasta (*.fas)|*.fas|");
//  strFileFormat += _T("Nexus (*.nex)|*.nex|");
//  strFileFormat += _T("Phylip (*.phy)|*.phy|");
//  strFileFormat += _T("Treecon (*.tre)|*.tre|");

  return strFileFormat;
}


bool MyFrame::startDist()
{
  int iFileType = rdoDistInput->GetSelection();

  wxString strTempFile;
  if ( chkDistInput->IsChecked() )
  {
    TempFile t(_T("distInput"));
    txtDistInput->SaveFile( t.file_name );
    strTempFile = t.file_name;
  }
  else
    strTempFile = mPnjFilePath;

  profdist::alignment sequences;
  try{
    switch(iFileType)  {
      case 0:parse_fasta( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 1:parse_embl( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 2:parse_bdb( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 3:parse_marna( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 4:parse_forester( convert_back( strTempFile.ToAscii() ), sequences );break;
    }

  }catch( std::exception const& e ) {

    size_t num_seq = sequences.size();
    if( num_seq >= 2 ) {
      std::ostringstream msg;
      msg << "Continue distance calculation?\n"
        << e.what() 
        << " but " << num_seq 
        << " Sequences found." << endl; 
      if( wxMessageBox(  convert(msg.str()), _T( "Continue Dinstance?" ), // caption
            wxYES_NO | wxICON_QUESTION ) != wxYES )
        return false;
    }
    else {
      error_output( this, e );
      return false;
    }
  }

  txtPnjOutput->Clear();


  size_t ip; 
  if( iFileType > 2 )
    ip = 2;
  else ip = is_proteine(); // is_proteine() asks the user which type of input is given.

  switch(ip) {
    case 0:
      if ( chkDistMultiple->IsChecked() )
        return startDistMultiple<profdist::rna_traits>( sequences);
      else
        return startDistSingle<profdist::rna_traits>( sequences);
      break;
    case 1:
      if ( chkDistMultiple->IsChecked() )
        return startDistMultiple<profdist::protein_traits>( sequences );
      else
        return startDistSingle<profdist::protein_traits>( sequences );
      break;
    case 2:
      if ( chkDistMultiple->IsChecked() )
        return startDistMultiple<profdist::rna_structure_traits>( sequences );
      else
        return startDistSingle<profdist::rna_structure_traits>( sequences );
      break;
  }
}

template<typename Traits>
bool MyFrame::startDistSingle( profdist::alignment & align )
{
  int iFileType;
  typename Traits::rate_matrix Q(0);

  if ( !loadRateMatrix<Traits>( Q ) )
    return false;

  AlignCode<Traits> alignCode;
  alignCode.read_sequences(align);
  align.clear();
  if( alignCode.get_num_sequences() <= 1 ) 
      throw runtime_error("not enough sequences for distance calculation found");


  std::size_t num_seq = alignCode.get_num_sequences();
  std::size_t max_steps = ( num_seq - 1 ) * ( num_seq - 2 ) / 2;

  ProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ),
      max_steps, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );

  distance_matrix matrix( num_seq, num_seq, 0.0 );

  try
  {
    compute_distance( alignCode, matrix, Q, mEvoModel,  /* profdist::Taylor */ mExpm , mFMax, dialog );
  }
  catch ( const error & e )
  {
    error_output( this, e );
    return false;
  }
  catch( const std::exception &e )
  {
    error_output( this, e );
    return false;
  }
  txtDistOutput->Clear();
  txtDistOutput->SetInsertionPoint( 0 );

  std::vector<std::string> seq_names( alignCode.get_num_sequences() );
  for( std::size_t i = 0; i < num_seq; ++i )
    seq_names[i] = alignCode.get_sequence_name( i );

  {
    wx_streambuf buf(*txtDistOutput);
    ostream out( &buf );
    print_distance_matrix( out, matrix, seq_names.begin(), seq_names.end() );
    out.flush();
  }
  move_to_top(txtDistOutput);
  return true;
}

template<typename Traits>
bool MyFrame::startDistMultiple( profdist::alignment & sequences )
{
  typename Traits::rate_matrix Q(0);
  if( !loadRateMatrix<Traits>(Q) )
    return false;

  int ft = rdoDistInput->GetSelection();
  size_t num_seq = sequences.size();
  size_t num_align = mNumberBoot;
  //size_t max_steps = num_align*(( num_seq/num_align - 1 ) * ( num_seq/num_align - 2 ) / 2);
  size_t max_steps = mNumberBoot;

  ProgressDialog first_dialog( _T( "Progress dialog" ), _T( "Ratematrix parsed, alignments parsed, calculating distances" ),
      max_steps, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );

  size_t step = 0;
  
  first_dialog.update(step, "");
  
  txtDistOutput->Clear();
  {
    wx_streambuf buf(*txtDistOutput);
    ostream out( &buf );
	ostringstream oss;

    try {
      while( ! sequences.empty() )
      {
        AlignCode<Traits> code;
        read_from_bootstrap( code, sequences, num_seq, 0,0 ); // use names to decide about stuff 
		
        distance_matrix matrix( code.get_num_sequences(), code.get_num_sequences(), 0.0 );
        compute_distance( code, matrix, Q, mEvoModel, mExpm, mFMax /*, first_dialog*/ );
		
        std::vector<std::string> seq_names( code.get_num_sequences() );
        for( std::size_t i = 0; i < code.get_num_sequences(); ++i )
          seq_names[i] = code.get_sequence_name( i );
		
        print_distance_matrix( /*out*/oss, matrix, seq_names.begin(), seq_names.end() );
		
        //step += code.get_num_sequences();
				step++;
				//bool go = first_dialog.update( first_dialog.get_actual_position(), "" );
				bool go = first_dialog.update( step, "" );
				if( ! go /*second_dialog.update( step )*/ )
				{
					if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
							wxYES_NO | wxICON_QUESTION ) == wxYES )
					{
						out << oss.str() << std::flush;
						first_dialog.Close();
						return false;
					}
					first_dialog.Resume();
				}
      }
    } catch ( const error & e )
    {
      error_output( this, e );
      first_dialog.Close();
      return false;
    }
    catch( const std::exception &e )
    {
      error_output( this, e );
      first_dialog.Close();
      return false;
    }

    first_dialog.Close();
	oss.flush();
	out << oss.str();
    out.flush();
  }
  move_to_top(txtDistOutput);
  return true;
}



void MyFrame::saveDist()
{
  wxString strFileFormat;
  strFileFormat = _T("Matrix (*.mat)|*.mat|");
  strFileFormat += _T("Text (*.txt)|*.txt|");
  strFileFormat += _T("All files (*)|*");

  wxFileDialog dialog( this, _T( "Save File Dialog" ), _T( "" ), _T( "" ), strFileFormat, wxSAVE | wxOVERWRITE_PROMPT );


  if ( dialog.ShowModal() == wxID_OK )
    txtDistOutput->SaveFile( dialog.GetPath() );
}


bool MyFrame::startNj()
{
  if ( chkNjMultiple->IsChecked() )
    return startNjMultiple();
  else
    return startNjSingle();
}

bool MyFrame::startNjMultiple()
{

  try{ 

    bool cont = true;
    std::size_t max = mNumberBoot - 1;

    txtNjOutput->Clear();
    {
      wx_streambuf buf( *txtNjOutput );
      ostream out( &buf );

      TempFile matrix_file( _T("matrix_temp") );

//       wxProgressDialog dialog( _T( "Progress dialog" ), _T( "" ),
//           mNumberBoot, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
	  ProgressDialog dialog( _T( "Progress dialog" ), _T( "Neighbour Joining" ), mNumberBoot, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );

      {
        txtNjInput->SaveFile( matrix_file.file_name );

        std::ifstream input( convert_back( matrix_file.file_name.ToAscii() ).c_str() );



        for ( std::size_t i = 0; i < mNumberBoot; ++i )
        {
          if ( i == max )
            cont = dialog.update( i, "Task finished!" );
          else
            cont = dialog.update( i );

          if ( !cont )
          {
            if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
                  wxYES_NO | wxICON_QUESTION ) == wxYES )
              return false;
            dialog.Resume();
          }

          vector<string> seq_name;
          distance_matrix matrix;
          read_distance_matrix( input, matrix, back_insert_iterator<vector<string> >( seq_name ) );

          if( ! input )
            throw runtime_error("Error parsing matrix, ");

          ProgressSink sink;
          bionj( matrix, seq_name, out, sink);
        }
      }

      wxMessageDialog dialog2( this, _T("Trees successful created!"), _T("Info"), wxICON_INFORMATION );

      out.flush();
    }
    move_to_top( txtNjOutput );

    return true;
  }catch ( const error & e ){
    error_output( this, e );
    return false;
  }catch ( exception const & e ){
    error_output( this, e );
    return false;
  }
}


bool MyFrame::startNjSingle()
{
  distance_matrix matrix;

  //bool cont = TRUE;
  vector<string> seq_names;

  if ( chkNjInput->IsChecked() )
  {
    TempFile t( _T("distInput") );
    txtNjInput->SaveFile( t.file_name );
    ifstream infile( convert_back( t.file_name.ToAscii() ).c_str() );

    read_distance_matrix( infile, matrix,  back_insert_iterator<vector<string> >(seq_names)  );

    if(! infile )
    {
      error_output( this,"Error parsing matrix" );
      return false;
    }
  }
  else
  {
    wxString volume, path, name, ext;
    wxFileName::SplitPath( mNjFilePath, & volume, & path, & name, & ext );

    ifstream infile( convert_back( mNjFilePath.ToAscii() ).c_str() );

    read_distance_matrix( infile, matrix,  back_insert_iterator<vector<string> >(seq_names)  );

    if(! infile )
    {
      error_output( this,"Error parsing matrix" );
      return false;
    }
  }


  int numMaxSteps = matrix.nRows() - 3;

  ProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ),
      numMaxSteps, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );


  txtNjOutput->Clear();
  {
    wx_streambuf buf( *txtNjOutput );
    ostream out( &buf );
    bionj( matrix, seq_names, out, dialog );

    wxMessageDialog dialog2( this, _T("Tree successful created!"), _T("Info"), wxICON_INFORMATION );
    out.flush();
  }
  move_to_top( txtNjOutput );

  return true;
}



void MyFrame::saveNj()
{

  wxFileDialog dialog( this, _T( "Save File Dialog" ), _T( "" ), _T( "" ),
      _T( "All files (*)|*" )
      , wxSAVE | wxOVERWRITE_PROMPT );


  if ( dialog.ShowModal() == wxID_OK )
    txtNjOutput->SaveFile( dialog.GetPath() );
}



bool MyFrame::startCons()
{
  try {
    txtConsOutput->Clear();
    wx_streambuf buf( *txtConsOutput );
    ostream out( &buf );

    TempFile cons_temp( _T("consense_temp") );
    txtConsInput->SaveFile( cons_temp.file_name );

    std::list<profdist::parsed_node::node_ptr> trees;
    std::vector<std::string> names;

    parse_newick( convert_back( cons_temp.file_name.ToAscii() ), trees, names );

    CountedSets cons;
    for( std::list<profdist::parsed_node::node_ptr>::const_iterator it = trees.begin(), e = trees.end(); it != e; ++it ) {
      profdist::get_splits( **it, cons, names.size());
    }

    cons.consense(names.size());
    Tree tree( cons, names.size(), trees.size());
    tree.print( out, names, trees.size() );
    out << std::endl;
    out.flush();
  } catch( std::exception const & e ) {
    error_output( this, e );
    return false;
  } catch( ... ) {
    error_output( this,"Unknown exception caught" );
    return false;
  }

  return true;
}



void MyFrame::saveCons()
{

  wxFileDialog dialog( this, _T( "Save File Dialog" ), _T( "" ), _T( "" ),
      _T( "All files (*)|*" )
      , wxSAVE | wxOVERWRITE_PROMPT );


  if ( dialog.ShowModal() == wxID_OK )
    txtConsOutput->SaveFile( dialog.GetPath() );
}

template<typename Traits>
bool MyFrame::loadRateMatrix( typename Traits::rate_matrix & Q )
{
  if ( mEvoModel > 1 )
  {
    ifstream infile( convert_back(mFileRateMatrix.ToAscii()).c_str() );

    if( !infile ) {
      wxMessageDialog dialog( this, _T("Ratematrix not found!"), _T( "Error!" ), wxICON_ERROR );
      dialog.ShowModal();
      return false;
    }

    if( read_rate_matrix<Traits>( infile, Q) && !infile.fail() )
        return true;
    
    wxMessageDialog dialog( this, _T("Ratematrix format does not fit to input data."), _T( "Error!" ), wxICON_ERROR );
    dialog.ShowModal();
    return false;
  }
  else return true;
}



bool MyFrame::startPnj()
{
  int iFileType;
  wxString strTempFile;


  iFileType = rdoPnjInput->GetSelection();

  if ( chkPnjInput->IsChecked() )
  {
    TempFile t(_T("pnjInput"));
    txtPnjInput->SaveFile( t.file_name );
    strTempFile = t.file_name;
  }
  else
    strTempFile = mPnjFilePath;

  profdist::alignment sequences;
  try{
    switch(iFileType)  {
      case 0:parse_fasta( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 1:parse_embl( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 2:parse_bdb( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 3:parse_marna( convert_back( strTempFile.ToAscii() ), sequences );break;
      case 4:parse_forester( convert_back( strTempFile.ToAscii() ), sequences );break;
    }

  }catch( std::exception const& e ) {

    size_t num_seq = sequences.size();
    if( num_seq >= 2 ) {
      std::ostringstream msg;
      msg << "Continue profile neighbour joining?\n"
        << e.what() 
        << " but " << num_seq 
        << " Sequences found." << endl; 
      if( wxMessageBox(  convert(msg.str()), _T( "Continue PNJ?" ), // caption
            wxYES_NO | wxICON_QUESTION ) != wxYES )
        return false;
    }
    else {
      error_output( this, e );
      return false;
    }
  }

  txtPnjOutput->Clear();

  size_t ip;
  if( iFileType > 2 )
    ip = 2;
  else 
    ip = is_proteine();

  if( iFileType != 1 )
    for( profdist::alignment::iterator it = sequences.begin(), e = sequences.end(); it != e; ++it )
      it->classification.push_back( it->id );

  if( mPnjShowOC == 1 )
  {
    ProfileDialog::profile_list profiles;
    ProfileDialog::name_list sequence_names;
    vector<string> profile_names;

    if ( ! create_group_dialog( sequences, profiles, profile_names, sequence_names ) ) 
      return false;
    else {

      switch(ip) {
        case 0: 
          {
            Profile profile( sequences, rna_traits(), profiles );
            sequences.clear();
            profdist::rna_traits::rate_matrix Q(0);
            if ( !loadRateMatrix<profdist::rna_traits>( Q ) )
              return false;
            return startPnjWithProfile<profdist::rna_traits>( profile, profiles, profile_names, sequence_names, Q );
          }
        case 1: 
          {
            Profile profile( sequences, protein_traits(), profiles );
            sequences.clear();
            profdist::protein_traits::rate_matrix Q(0);
            if ( !loadRateMatrix<profdist::protein_traits>( Q ) )
              return false;
            return startPnjWithProfile<profdist::protein_traits>( profile, profiles, profile_names, sequence_names, Q );
          }
        case 2:
          {
            Profile profile( sequences, rna_structure_traits(), profiles );
            sequences.clear();
            profdist::rna_structure_traits::rate_matrix Q(0);
            if ( !loadRateMatrix<profdist::rna_structure_traits>( Q ) )
              return false;
            return startPnjWithProfile<profdist::rna_structure_traits>( profile, profiles, profile_names, sequence_names, Q );
          }
      }
    }
  }
  else
  {
    switch(ip) {
      case 0: {
                AlignCode<profdist::rna_traits> code;
                code.read_sequences( sequences );
                sequences.clear();
                profdist::rna_traits::rate_matrix Q(0);
                if ( !loadRateMatrix<profdist::rna_traits>( Q ) )
                  return false;
                return startPnjWithAlignment<profdist::rna_traits>( code, Q );
              }

      case 1: {
                AlignCode<profdist::protein_traits> code;
                code.read_sequences( sequences );
                sequences.clear();
                profdist::protein_traits::rate_matrix Q(0);
                if ( !loadRateMatrix<profdist::protein_traits>( Q ) )
                  return false;
                return startPnjWithAlignment<profdist::protein_traits>( code, Q );
              }
      case 2: {
                AlignCode<profdist::rna_structure_traits> code;
                code.read_sequences( sequences );
                sequences.clear();
                profdist::rna_structure_traits::rate_matrix Q(0);
                if ( !loadRateMatrix<profdist::rna_structure_traits>( Q ) )
                  return false;
                return startPnjWithAlignment<profdist::rna_structure_traits>( code, Q );
              }
    }
  }

}

template<typename Traits>
bool MyFrame::startPnjWithProfile( Profile & profile, ProfileDialog::profile_list const& profiles,
    vector<string> const& profile_names, ProfileDialog::name_list const& sequence_names, typename Traits::rate_matrix const& Q )
{
  try
  {

    {
      wx_streambuf buf( *txtPnjOutput );
      ostream out( &buf );
#ifdef DANIEL_CODE
      ofstream daniel("daniel", ios::app);
#endif

      tree_types::profile_map profiles_found;

#ifdef DANIEL_CODE
      std::ofstream foo("profile_foo");
      foo << profile;
#endif

      std::size_t num_profiles = profile.get_num_profiles();
      distance_matrix matrix( num_profiles, num_profiles, 0.0 );
      compute_distance<Traits>( profile, matrix, Q, mEvoModel, /*profdist::Taylor*/ mExpm, mFMax );
#ifdef DANIEL_CODE
      foo << matrix;
#endif

#ifdef DANIEL_CODE
      {
        daniel << matrix << std::endl;
        daniel << "Profile:" << std::endl;

        for( std::size_t prof_index = 0; num_profiles != prof_index; ++prof_index ) 
          daniel << prof_index << ", " << profile_names[prof_index]  << std::endl;
        daniel.close();
      }
#endif
      ProgressSink sink;
      bionj( matrix, profile_names, out, sink );

      srand( time( NULL ) );
      std::size_t max = std::max( mNumberBoot, 1L ) + 3;
      std::size_t mini_step = 0;
//       wxProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ),
//           max, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
	  ProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ),
           max, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );

      { // life time of cons and tree:
        CountedSets cons;
        if( mNumberBoot )
        {
          for( std::size_t i = 0; i != mNumberBoot; ++i )
          {
            if( ! dialog.update( mini_step++ ) )
            {
              if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
                    wxYES_NO | wxICON_QUESTION ) == wxYES )
                return false;
              dialog.Resume();
            }
            Profile profile_b( profile.get_num_sites(), profile.get_num_profiles() );
            bootstrap( profile, profile_b );
            try {
              compute_distance<Traits>( profile_b, matrix, Q, mEvoModel, /*profdist::Taylor*/ mExpm, mFMax );
            } catch( ... ) {
              std::cout << "Failing Profile was: :- \n" << profile_b << std::endl;
              throw;
            }
            bionj( matrix, cons, sink );
          }
        }
        else 
        {
          compute_distance<Traits>( profile, matrix, Q, mEvoModel,  /* profdist::Taylor */ mExpm, mFMax );
          bionj( matrix, cons, sink);
          dialog.update( mini_step++);
        }

        dialog.update( mini_step++, "Creating consense of all trees" );
        cons.consense( num_profiles );

        dialog.update( mini_step++, "Rebuilding tree" );
        Tree tree( cons, num_profiles, mNumberBoot);

        cons.clear();

        tree.print( out, profile_names, mNumberBoot );
        out << std::endl;

        out.flush();

        dialog.update( mini_step++, "Searching for new profiles" );
        profdist::profile_set profiles;
        if ( mPnjMethod < 3 )
        {
          show_tree( profile_names, tree );
          if ( !create_profile_dialog( profiles ) )
            return false;
        }

        // search for Profiles in the tree
        size_t threshold = std::size_t( float(mPnjBootstrapThreshold * mNumberBoot) / 100.0f );
        if( mPnjBootstrapThreshold > 100 ) 
          threshold = mNumberBoot + 1;

        profdist::identical_seq_set ident; 
        profile.get_identical_sequences( ident, float(mPnjIdentityThreshold ) / 100.0f);
        tree.find_profile_first( profiles_found, profiles, ident, threshold, mPnjMethod );
        //tree.find_profile( profiles_found, profiles, ident, threshold, mPnjMethod );

      }
      // create first Time Profiles with Alignment Code an MapTree

      dialog.update( mini_step++, /*_T(*/ "Task finished" /*)*/ );
      dialog.Close();

      if( profiles_found.size() != num_profiles ) {
        profile.refine( profiles_found );
        start_pnj_main_loop<Traits>( profile, profile_names, profiles_found, Q, out );
      }
      out.flush();
    }

    move_to_top( txtPnjOutput );


    return true;
  }
  catch ( const error & e )
  {
    error_output( this, e );
    return false;
  }
  catch( const std::exception &e )
  {
    error_output( this, e );
    return false;
  }

}

template<typename Traits>
bool MyFrame::start_pnj_main_loop( Profile & profile, std::vector<std::string> const& seq_names, tree_types::profile_map & profiles_found, typename Traits::rate_matrix const& Q, ostream & out ) 
{
  try {
    wx_streambuf buf( *txtPnjOutput );
    ostream out( &buf );

#define DEBUGOR std::cout << __FILE__ << ':' << __LINE__ << std::endl;
    int max = std::max(mNumberBoot,1L)+ 4;
    // getNumber
    std::size_t num_profiles = profile.get_num_profiles();
    std::size_t num_new_profiles = 1;
    std::size_t step = 1;
    while( num_new_profiles && num_profiles > 3 )
    {
      std::size_t mini_step = 0;
      wxString dialog_text( _T( "Duration of step: " ) );
      dialog_text << ++step;
//       wxProgressDialog dialog( _T( "Progress dialog" ),
//           dialog_text, max, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
	  ProgressDialog dialog( _T( "Progress dialog" ),
          dialog_text, max, this, wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME ); 

      distance_matrix matrix( num_profiles, num_profiles, 0.0 );
#ifdef DANIEL_CODE
      {
        ofstream daniel( "daniel", std::ios::app);
        compute_distance<Traits>( profile , matrix, Q, mEvoModel,  /*profdist::Taylor */ mExpm, mFMax );
        daniel << matrix << std::endl;
        daniel << "Profile:" << std::endl;
        std::size_t prof_index = 0;
        for( tree_types::profile_map::const_iterator it = profiles_found.begin(), e = profiles_found.end();  it != e; ++it, ++prof_index ) {
          daniel << prof_index;
          if( it->second->get_split_set().empty() )
            daniel << ", Profile " << it->second->get_reference_position() << " from last round.";
          else {
            daniel << ", composed by previous profiles: ";
            for( Node::set_type::const_iterator s_it = it->second->get_split_set().begin(), s_e = it->second->get_split_set().end(); s_it != s_e; ++s_it )
              daniel << *s_it << ", ";
          }
          daniel << std::endl;
        }
        daniel.close();
      }
#endif


      CountedSets split_sets;
      if( mNumberBoot )
      {
        for( int i = 0; i < mNumberBoot; ++i )
        {
          if( ! dialog.update( mini_step++ ) )
          {
            if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
                  wxYES_NO | wxICON_QUESTION ) == wxYES )
              return false;
            dialog.Resume();
          }

          Profile other_profile( profile.get_num_sites(), profile.get_num_profiles() );
          bootstrap( profile, other_profile );
          try {
            compute_distance<Traits>( other_profile, matrix, Q, mEvoModel,  /*profdist::Taylor */  mExpm, mFMax );
          } catch( ... ) {
            std::cout << "Failing Profile was: :- \n" << other_profile << std::endl;
            throw;
          }

          ProgressSink sink;
          bionj( matrix, split_sets, sink );
        }
      }
      else 
      {
        compute_distance<Traits>( profile, matrix, Q, mEvoModel, /*profdist::Taylor*/ mExpm, mFMax );

        ProgressSink sink;
        bionj( matrix, split_sets, sink );
        dialog.update( mini_step++ );
      }


      dialog.update( mini_step++, "Creating consense of all trees" );
      split_sets.consense( num_profiles );
      
      dialog.update( mini_step++, "Rebuilding tree" );
      Tree tree( split_sets, num_profiles, mNumberBoot);

      split_sets.clear();

#ifdef PROFDIST_DEBUG
     {
        ostringstream name;
        name << "tree_from_consense_" << step << ".dot";
        ofstream debug( name.str().c_str() );
        if( debug )
          tree.print_graphviz_debug( debug, seq_names );
      }
      
      {
        ostringstream name;
        name << "profiles_found_" << step << ".dot";
        ofstream debug( name.str().c_str() );
        if(debug) {
          debug << "digraph {\n node [shape=Mrecord]; \n";
          for( tree_types::profile_map::const_iterator it = profiles_found.begin(), e = profiles_found.end();  it != e; ++it )
            if( it->second ) 
              it->second->print_graphviz_debug( debug, seq_names );
            else 
              std::cerr << "EEK" << std::endl;
          debug << "}\n";
        }
      }
#endif

      tree.union_tree( profiles_found );
      tree.print( out, seq_names, mNumberBoot );
      out << std::endl;
#ifdef PROFDIST_DEBUG
      {
        ostringstream name;
        name << "tree_from_union_" << step << ".dot";
        ofstream debug( name.str().c_str() );
        if( debug )
          tree.print_graphviz_debug( debug, seq_names );
      }
#endif

      dialog.update( mini_step++, "Searching for new profiles" );
      profdist::profile_set profiles;

      if ( mPnjMethod < 2 )
      {
        show_tree( seq_names, tree );
        if ( !create_profile_dialog( profiles ) )
          return false;
      }
      profiles_found.clear();

      size_t threshold = std::size_t( float(mPnjBootstrapThreshold * mNumberBoot) / 100.0f );
      if( mPnjBootstrapThreshold > 100 ) 
        threshold = mNumberBoot + 1;

      {
        profdist::identical_seq_set ident; 
        profile.get_identical_sequences( ident, float(mPnjIdentityThreshold ) / 100.0f);
        tree.find_profile( profiles_found, profiles, ident, threshold, mPnjMethod );
      }
      
      dialog.update( mini_step++, "Rebuilding set of profiles" );
      profile.refine( profiles_found );

      num_new_profiles = num_profiles - profile.get_num_profiles();
      num_profiles = profile.get_num_profiles();
      dialog.update( mini_step++, "Task finished" );
      dialog.Close();
    }
    out.flush();
    return true;
  }
  catch( const error & e )
  {
    error_output( this, e );
    return false;
  }
  catch( const std::exception &e )
  {
    error_output( this, e );
    return false;
  }
}

template<typename Traits>
bool MyFrame::startPnjWithAlignment( AlignCode<Traits> & code, typename Traits::rate_matrix & Q )
{
  try
  {
    int max = std::max( mNumberBoot, 1L ) + 3;

    std::size_t num_sequences = code.get_num_sequences();
    tMatrix<double> matrix( num_sequences, num_sequences, 0.0 );


//     wxProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ), max, // range
//         this, // parent
//         wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | //-- try this as well
//         wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
	ProgressDialog dialog( _T( "Progress dialog" ), _T( "Duration of step: 1" ), max, // range
        this, // parent
        wxPD_CAN_ABORT | wxPD_APP_MODAL | wxPD_AUTO_HIDE | //-- try this as well
        wxPD_ELAPSED_TIME | wxPD_REMAINING_TIME );
    std::size_t mini_step = 0;


    compute_distance<Traits>( code, matrix, Q, mEvoModel, /*profdist::Taylor */  mExpm, mFMax );

    // get Sequence Names
    vector<string> seq_names = code.get_sequence_names();

    profdist::identical_seq_set ident; 
    code.get_identical_sequences( ident, float(mPnjIdentityThreshold ) / 100.0f);

    txtPnjOutput->Clear();
    {
      wx_streambuf buf( *txtPnjOutput );
      ostream out( &buf );
      ProgressSink sink;
      bionj( matrix, seq_names, out, sink );

      srand( time( NULL ) );

      tree_types::profile_map profiles_found;

      { // life time of consense and tree:

        CountedSets consense;

        if( mNumberBoot )
        {
          for ( int i = 0; i < mNumberBoot; ++i )
          {
            if( !dialog.update( mini_step++ ) )
            {
              if ( wxMessageBox( _T( "Do you really want to cancel?" ), _T( "Progress dialog question" ), // caption
                    wxYES_NO | wxICON_QUESTION ) == wxYES )
                return false;
              dialog.Resume();
            }
            AlignCode<Traits> alignBootstrap; 

            bootstrap( code, alignBootstrap );

            compute_distance( alignBootstrap, matrix, Q, mEvoModel, /*profdist::Taylor */ mExpm, mFMax );

            ProgressSink sink;
            bionj( matrix, consense, sink);
          }
        }
        else 
        {
          compute_distance( code, matrix, Q, mEvoModel, /*profdist::Taylor */ mExpm, mFMax );
          ProgressSink sink;
          bionj( matrix, consense, sink);
          dialog.update( mini_step++ );
        }


        dialog.update( mini_step++, "Creating consense of all trees" );
        consense.consense( num_sequences );

        dialog.update( mini_step++, "Rebuilding tree" );
        Tree tree( consense, num_sequences, mNumberBoot);

        consense.clear();


        tree.print( out, code.get_sequence_names(), mNumberBoot );
        out << std::endl;
        out.flush();
#ifdef PROFDIST_DEBUG
        {
          ofstream debug( "tree_from_consense_0.dot" );
          if( debug )
            tree.print_graphviz_debug( debug, seq_names );
        }
#endif

        profdist::profile_set profiles;

        dialog.update( mini_step++, "Searching new profiles" );
        // if pnjMethod == manual or == half-automatic then a Dialog for Profile input will be created
        if ( mPnjMethod < 3 )
        {
          show_tree( seq_names, tree );
          if ( !create_profile_dialog( profiles ) )
            return false;
        }

        // search for Profiles in the tree
        size_t threshold = std::size_t( float(mPnjBootstrapThreshold * mNumberBoot) / 100.0f );
        if( mPnjBootstrapThreshold > 100 ) 
          threshold = mNumberBoot + 1;
        tree.find_profile_first( profiles_found, profiles, ident, threshold, mPnjMethod );
        //tree.find_profile( profiles_found, profiles, ident, threshold, mPnjMethod );
#ifdef PROFDIST_DEBUG
        {
          ofstream debug( "after_profiles_found.dot" );
          if( debug )
            tree.print_graphviz_debug( debug, seq_names );
        }

        {
          ofstream debug("profiles_found_0.dot" );
          if(debug) {
            debug << "digraph {\n node [shape=Mrecord]; \n";
            for( tree_types::profile_map::const_iterator it = profiles_found.begin(), e = profiles_found.end();  it != e; ++it ) {
              if( it->second ) 
                it->second->print_graphviz_debug( debug, seq_names );
              else 
                std::cerr << "EEK" << std::endl;
            }
            debug << "}\n";
          }
        }
#endif


      }

      dialog.update( mini_step++, "Task finished" );
      dialog.Close();

      if( profiles_found.size() != code.get_num_sequences() ) {
        Profile profile( code, profiles_found );
        // free allocated memory from alignment code
        code.clear();
        start_pnj_main_loop<Traits>( profile, seq_names, profiles_found, Q, out );
      }

      out.flush();
    }
    move_to_top( txtPnjOutput );
    return true;
  }
  catch ( const error & e )
  {
    error_output( this, e );
    return false;
  }
  catch( const std::exception &e )
  {
    error_output( this, e );
    return false;
  }

}

bool MyFrame::loadXmlFile()
{
  wxConfigBase* config = wxConfigBase::Get( true );

  long em, expm, fmax;

  config->Read(_T("/profdist/bootstrap_number"), &mNumberBoot, 100);
  config->Read(_T("/profdist/evolutionary_model"), &em, 0 );
  switch( em ) { 
    case 0: mEvoModel = profdist::Jukes; break;
    case 1: mEvoModel = profdist::Kimura; break;
    case 2: mEvoModel = profdist::GTR; break;
    case 3: mEvoModel = profdist::LogDet; break;
    default: mEvoModel = profdist::Jukes;
  }

  config->Read(_T("/profdist/matrix_exponential"), &expm, 1);
  switch(expm) {
    case 0: mExpm = profdist::Pade; break;
    case 1: mExpm = profdist::Taylor; break;
		case 2: mExpm = profdist::Eigenvalue; break;
    default: mExpm = profdist::Taylor;
  }
  config->Read(_T("/profdist/function_maximation"), &fmax, 1 );
  switch(fmax) {
    case 0: mFMax = profdist::Derivation; break;
    case 1: mFMax = profdist::NewtonMethod; break;
    case 2: mFMax = profdist::Robust; break;
    case 3: mFMax = profdist::Parabolic; break;
    default: mFMax = profdist::NewtonMethod;
  }


  config->Read(_T("/profdist/rate_matrix"), &mFileRateMatrix, convert(MyApp::get_examples_path()) + _T("/Q_arthro.txt")) ;
  config->Read(_T("/profdist/pnj/method"), &mPnjMethod, 3);
  config->Read(_T("/profdist/pnj/boostrap_threshold"), &mPnjBootstrapThreshold, 75);
  config->Read(_T("/profdist/pnj/identiy_threshold"), &mPnjIdentityThreshold, 95);
  config->Read(_T("/profdist/pnj/show_OC"), &mPnjShowOC, false);
  config->Read(_T("/profdist/treeviewer/view_type"), &mTreeViewType, 0);
  config->Read(_T("/profdist/treeviewer/binary_path_1"), &mFileTreeViewer1, wxString(_T("")));
  config->Read(_T("/profdist/treeviewer/binary_path_2"), &mFileTreeViewer2, wxString(_T("")));


  config->Read(_T("/profdist/gui/notebook_selection"), &mNtbSelection, 0);
  config->Read(_T("/profdist/gui/boot_input"), &mRdoBootInput, 0);
  config->Read(_T("/profdist/gui/boot_output"), &mRdoBootOutput, 0);
  config->Read(_T("/profdist/gui/dist_input"), &mRdoDistInput, 0);
  config->Read(_T("/profdist/gui/dist_output"), &mRdoDistOutput, 0);
  config->Read(_T("/profdist/gui/nj_input"), &mRdoNjInput, 0);
  config->Read(_T("/profdist/gui/nj_output"), &mRdoNjOutput, 0);
  config->Read(_T("/profdist/gui/cons_output"), &mRdoConsOutput, 0);
  config->Read(_T("/profdist/gui/pnj_input"), &mRdoPnjInput, 1);
  config->Read(_T("/profdist/gui/pnj_output"), &mRdoPnjOutput, 0);

  bool val;
  config->Read(_T("/profdist/gui/chk_boot_input"),  &val, true );
  chkBootInput->SetValue( val );
  txtBootInput->Enable( val );
  rdoBootInput->Enable( val );

  config->Read(_T("/profdist/gui/chk_boot_output"), &val, true );
  chkBootOutput->SetValue( val );
  txtBootOutput->Enable( val );
  rdoBootOutput->Enable( val );

  config->Read(_T("/profdist/gui/chk_dist_input"),  &val, true );
  chkDistInput->SetValue( val );
  txtDistInput->Enable( val );
  rdoDistInput->Enable( val );

  config->Read(_T("/profdist/gui/chk_dist_output"), &val, true );
  chkDistOutput->SetValue( val );
  txtDistOutput->Enable( val );
  rdoDistOutput->Enable( val );

  config->Read(_T("/profdist/gui/chk_nj_input"),    &val, true );
  chkNjInput->SetValue( val );
  txtNjInput->Enable( val );
  rdoNjInput->Enable( val );

  config->Read(_T("/profdist/gui/chk_nj_output"),   &val, true );
  chkNjOutput->SetValue( val );
  txtNjOutput->Enable( val );
  rdoNjOutput->Enable( val );

  config->Read(_T("/profdist/gui/chk_pnj_input"),   &val, true );
  chkPnjInput->SetValue( val );
  txtPnjInput->Enable( val );
  rdoPnjInput->Enable( val );

  config->Read(_T("/profdist/gui/chk_pnj_output"),  &val, true );
  chkPnjOutput->SetValue( val );
  txtPnjOutput->Enable( val );
  rdoPnjOutput->Enable( val );


  return true;
}


void MyFrame::show_tree( vector<string> const& seq_names, Tree const& tree ) 
{
  if ( mTreeViewType < 2 )
  {
    TempFile t(_T("pnjTree") );
    ofstream out( convert_back(t.file_name.ToAscii()).c_str() );
    if( out ) {
      tree.print( out, seq_names, mNumberBoot);
      out << std::endl;
      out.close();

      display_file( t.file_name );
    }
    else 
      error_output( this, "Error displaying file, could not access directory for temporary files" );
  }
}





void MyFrame::display_file( wxString const& tempfile ) const
{
#ifdef DARWIN_SYSTEM
  switch(mTreeViewType) {
    case 0: 
    case 1: 
      {
        LSLaunchFSRefSpec launcher_spec;
        FSRef application;
        FSRef file;
        wxString app = (mTreeViewType==0)?mFileTreeViewer1:mFileTreeViewer2;
        FSPathMakeRef( reinterpret_cast<UInt8 const*>(tempfile.c_str()), &file, 0 );
        FSPathMakeRef( reinterpret_cast<UInt8 const*>(app.c_str()), &application, 0 );

        launcher_spec.appRef = &application;
        launcher_spec.numDocs = 1;
        launcher_spec.itemRefs = &file;
        launcher_spec.passThruParams = 0;
        launcher_spec.launchFlags = kLSLaunchDefaults;
        launcher_spec.asyncRefCon = 0;
        LSOpenFromRefSpec( &launcher_spec, 0 );
        break;
      }
    default: 
      wxExecute( "java -jar htnewick.jar " + tempfile);
      break;
  }


#else
  wxString execute;

  if ( mTreeViewType == 0 ) // Classic Tree Viewer1
    execute = mFileTreeViewer1 + _T(" \"") + tempfile + _T("\"");
  else if ( mTreeViewType == 1 ) // Classic Tree Viewer2
    execute = mFileTreeViewer2 + _T(" \"") + tempfile + _T("\"");
  else // hypertree
    execute = _T("java -jar htnewick.jar ") + tempfile;

  wxExecute( execute );
#endif

}

void MyFrame::treeViewPnj()
{
  if ( mTreeViewType < 2 && txtPnjOutput->GetLastPosition() > 0 )
  {
    TempFile t(_T("pnjTree") );
    txtPnjOutput->SaveFile( t.file_name );

    display_file( t.file_name );
  }
}



wxTextCtrl * MyFrame::getFocusedText()
{
  wxWindow * win = FindFocus();

  wxTextCtrl * text = win ? wxDynamicCast( win, wxTextCtrl ) : NULL;
  return text; // ? text : textDef;
}

void MyFrame::move_to_top( wxTextCtrl * t )
{
  t->SetFocusFromKbd();
  t->SetInsertionPoint(0);
  t->ShowPosition(0);
}

void MyFrame::treeViewNj()
{

  if ( mTreeViewType < 2 && txtNjOutput->GetLastPosition() > 0 )
  {
    TempFile t(_T("pnjTree"));
    txtNjOutput->SaveFile( t.file_name );

    display_file( t.file_name );
  }
}



void MyFrame::treeViewCons()
{
  wxString execute;

  if ( mTreeViewType < 2 && txtConsOutput->GetLastPosition() > 0 )
  {
    TempFile t(_T("pnjTree"));
    txtConsOutput->SaveFile( t.file_name );

    display_file( t.file_name );
  }
}

